package infomat.algorithms;

import infomat.vectorspace.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.*;
/*
import java.util.Vector; 
import java.util.Map.Entry; 
import java.util.HashMap; 
import java.util.HashSet; 
import java.util.Set; 
*/

/**
 * KMeans
 *      
 * @version          070519
 * @author           Magnus Rosell
 */
public class KMeans extends GroupingAlgorithm{ 

	/*
    protected Properties properties;
    protected UniqueIdentifier identifier;
	*/

	public static int kmeansClusteringNumber = 1;
	
	public KMeans(){
		properties = new mro.util.properties.Properties("KMeans Properties");
		
		try{

			/*
			Property p = new Property("Dimension to cluster", "Rows", "To cluster rows or columns");
			PropertyValueDescription pvd = new PropertyValueDescription("Rows", "Cluster Rows");
			p.addValueDescription(pvd);
			pvd = new PropertyValueDescription("Columns", "Cluster Columns");
			p.addValueDescription(pvd);			
			properties.addProperty(p);
			 */
			
			Property p = new Property("Number of clusters", 5, "Number of clusers");
			properties.addProperty(p);

			p = new Property("Init Type", 1, "Kind of initialization");
			PropertyValueDescription pvd = new PropertyValueDescription("0", "\"Number of clusters\" random texts as start clusters");
			p.addValueDescription(pvd);
			pvd = new PropertyValueDescription("1", "Random clustering as start clusters.");
			p.addValueDescription(pvd);
			properties.addProperty(p);

			
			p = new Property("Repeat Type", 0, "Iteration criterion");
			pvd = new PropertyValueDescription("0", "Repeat \"Repeat number of times\" number of times");
			p.addValueDescription(pvd);
			properties.addProperty(p);
			
			p = new Property("Repeat number of times", 10, "If RepeatType=0");
			properties.addProperty(p);

			p = new Property("SimSort", true, "Sort objects in clusters according to similarity to cluster centroid.");
			pvd = new PropertyValueDescription(true, "Do it.");
			p.addValueDescription(pvd);
			pvd = new PropertyValueDescription(false, "Do not do it.");
			p.addValueDescription(pvd);
			properties.addProperty(p);

			
		}catch(PropertyNameConflictException pnce){
			// Won't happen here
		}
	}
	
	public KMeans(mro.util.properties.Properties properties){
		this.properties=properties;
	}
	
	
	public IObjectGrouping cluster(IMatrix matrix, IObjectSet ioset){
		
		// Dimension
		/*
		IObjectSet ioset = null;
		String dim = (String) properties.getPropertyValue("Dimension to cluster");
		if(dim.equalsIgnoreCase("Rows"))
			ioset=matrix.getRowIObjectSet();
		else if(dim.equalsIgnoreCase("Columns"))
			ioset=matrix.getColumnIObjectSet();
		else{
			System.out.println("Unknown dimension to cluster!");
			System.exit(0);			
		}
		*/
		
		
		// Init
		Vector<IObjectGroup> groups = init(matrix, ioset);
		
		// Repeat
		int repeatType = (Integer) properties.getPropertyValue("Repeat Type");
		if(repeatType == 0){
			int  repeat = (Integer) properties.getPropertyValue("Repeat number of times");
			for(int nr=0; nr<repeat; nr++){
				//System.out.println("repeat: "+repeat);
				printGroupSizes(groups);
				Vector<HashMap<IObject, Float>> centroids = new Vector<HashMap<IObject, Float>>();
				
				for(int g=0; g<groups.size(); g++){
					IObjectGroup group = groups.elementAt(g);
					HashMap<IObject, Float> centroid = group.getRepresentation(matrix);
					centroids.add(centroid);
					group.removeAll();
				}
				
				for(int o=0; o<ioset.numberOfIObjects(); o++){
					IObject io = ioset.getIObjectAt(o);
					HashMap<IObject, Float> representation = io.getRepresentation(matrix);
					float bestSim=0f;
					int bestC=0;
					for(int c=0; c<centroids.size(); c++){
						HashMap<IObject, Float> centroid = centroids.elementAt(c);
						float sim = Sorter.compareObjectToCentroid(representation, centroid);
						if(sim>bestSim){
							bestSim=sim;
							bestC=c;
						}
					}
					IObjectGroup group = groups.elementAt(bestC);
					group.add(io);
					
				}
				
			}
				
		
		}
		
		boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
		if(simSort){
			Sorter.sortInSimOrderWithinClusters(matrix, groups);
		}
		
		
		
		String name = "KMeans Clustering "+kmeansClusteringNumber;
		IObjectGrouping grouping = new IObjectGrouping(name);
		int i=1;
		while(!ioset.setGrouping(grouping)){
			grouping.setName(name+" ("+i+")");
			i++;
		}

		for(int g=0; g<groups.size(); g++){
			IObjectGroup group = groups.elementAt(g);
			grouping.add(group);
			printGroup(group);
		}
		
		kmeansClusteringNumber++;		
		return grouping;
	}
	
	
	private Vector<IObjectGroup> init(IMatrix matrix, IObjectSet ioset){
		int numOfClusters = (Integer) properties.getPropertyValue("Number of clusters");
		if(numOfClusters > ioset.numberOfIObjects()){
			System.out.println("Number of clusters exeeds the number of objects!");
			System.exit(0);
		}
		
		Vector<IObjectGroup> groups = new Vector<IObjectGroup>(); 
		int init = (Integer) properties.getPropertyValue("Init Type");
		if(init==0){
			HashSet<IObject> ios = new HashSet<IObject>();
			int c=0;
			while(c<numOfClusters){
				IObject io = null;
				boolean goOn=true;
				while(goOn){
					int r = (int) (Math.random()*ioset.numberOfIObjects());
					io = ioset.getIObjectAt(r);
					//System.out.println("io: "+io.getString());
					goOn=ios.contains(io);
					/*
					try{
						Thread.sleep(10);
					}catch(InterruptedException ie){
						// Do nothing
					}
					*/
				}
				ios.add(io);
				IObjectGroup group = new IObjectGroup("Cluster "+(c+1));
				group.add(io);
				groups.add(group);
				c++;
				//System.out.println("c: "+c);
			}
		}else if(init==1){
			IObjectGrouping grouping = new RandomClusterer().cluster(matrix, ioset);
			groups = grouping.getGroups();
			
			
			
		}
		return groups;	
	}
	
	
	
	
	private void printGroupSizes(Vector<IObjectGroup> groups){
		for(int i=0; i<groups.size(); i++){
			System.out.print(groups.elementAt(i).numberOfIObjects()+" ");
		}
		System.out.println();
	}
	
	
	private void printGroup(IObjectGroup group){
		System.out.println(group.getString());
		Vector ordered = group.getOrdered();
		for(int i=0; i<ordered.size(); i++)
			System.out.println(" * "+((IObject) ordered.elementAt(i)).getString());
		
	}
	
	
	
	
    public String getInfoString(){
    	return "KMeans";
    }

    public String getAlgorithmName(){
    	return "KMeans";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
