package infomat.algorithms;

import infomat.vectorspace.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.*;
/*
import java.util.Vector; 
import java.util.Map.Entry; 
import java.util.HashMap; 
import java.util.HashSet; 
import java.util.Set; 
*/

/**
 * KMeans
 *      
 * @version          070519
 * @author           Magnus Rosell
 */
public class RandomClusterer extends GroupingAlgorithm{ 

	/*
    protected Properties properties;
    protected UniqueIdentifier identifier;
	*/

	private static int randomClusteringNumber = 1;
	
	public RandomClusterer(){
		properties = new mro.util.properties.Properties("Random Clustering Properties");
		
		try{

			/*
			Property p = new Property("Dimension to cluster", "Rows", "To cluster rows or columns");
			PropertyValueDescription pvd = new PropertyValueDescription("Rows", "Cluster Rows");
			p.addValueDescription(pvd);
			pvd = new PropertyValueDescription("Columns", "Cluster Columns");
			p.addValueDescription(pvd);			
			properties.addProperty(p);
			 */
			
			Property p = new Property("Number of clusters", 5, "Number of clusers");
			properties.addProperty(p);

			
		}catch(PropertyNameConflictException pnce){
			// Won't happen here
		}
	}
	
	public RandomClusterer(mro.util.properties.Properties properties){
		this.properties=properties;
	}
	
	
	public IObjectGrouping cluster(IMatrix matrix, IObjectSet set){
		
		// Dimension
		IObjectSet ioset = set;

		/*
		IObjectSet ioset = null;
		String dim = (String) properties.getPropertyValue("Dimension to cluster");
		if(dim.equalsIgnoreCase("Rows"))
			ioset=matrix.getRowIObjectSet();
		else if(dim.equalsIgnoreCase("Columns"))
			ioset=matrix.getColumnIObjectSet();
		else{
			System.out.println("Unknown dimension to cluster!");
			System.exit(0);			
		}
		*/
		
		// Init
		int numOfClusters = (Integer) properties.getPropertyValue("Number of clusters");
		if(numOfClusters > ioset.numberOfIObjects()){
			System.out.println("Number of clusters exeeds the number of objects!");
			System.exit(0);
		}
		
		Vector<IObjectGroup> groups = new Vector<IObjectGroup>(); 
		for(int c=0; c<numOfClusters; c++)
			groups.add(new IObjectGroup("Cluster "+(c+1)));
		for(int o=0; o<ioset.numberOfIObjects(); o++){
			IObject io = ioset.getIObjectAt(o);
			int gr = (int) (Math.random() * groups.size());
			IObjectGroup group = groups.elementAt(gr);
			group.add(io);
		}

		String name = "Random Clustering "+randomClusteringNumber;
		IObjectGrouping grouping = new IObjectGrouping(name);
		int i=1;
		while(!ioset.setGrouping(grouping)){
			grouping.setName(name+" ("+i+")");
			i++;
		}

		for(int g=0; g<groups.size(); g++){
			IObjectGroup group = groups.elementAt(g);
			grouping.add(group);
			//printGroup(group);
		}
		
		randomClusteringNumber++;		
		return grouping;
	}

	
	/*
	private void printGroup(IObjectGroup group){
		System.out.println(group.getString());
		Vector ordered = group.getOrdered();
		for(int i=0; i<ordered.size(); i++)
			System.out.println(" * "+((IObject) ordered.elementAt(i)).getString());
		
	}
	
	
	private float compareObjectToCentroid(HashMap<IObject, Float> representation,
			HashMap<IObject, Float> centroid){
		float sim =0f;
		Set<Map.Entry<IObject, Float>> set = representation.entrySet();
		Iterator<Map.Entry<IObject, Float>> it = set.iterator();
		while(it.hasNext()){
			Map.Entry<IObject, Float> entry = it.next();
			Float f = centroid.get(entry.getKey());
			if(f!=null){
				sim+= f * entry.getValue();
			}
		}
		return sim;
	}
	*/
	
    public String getInfoString(){
    	return "Random Clusterer";
    }

    public String getAlgorithmName(){
    	return "Random Clusterer";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
