package infomat.algorithms;

import infomat.vectorspace.*;

import java.util.*;

import mro.util.UniqueIdentifier;
import mro.util.properties.Property;
import mro.util.properties.PropertyNameConflictException;
import mro.util.properties.PropertyNotFoundException;
import mro.util.properties.PropertyValueDescription;

public class RelativeClusterer extends GroupingAlgorithm{

	private static int relativeClusteringNumber=1;
	
	public RelativeClusterer(){
		properties = new mro.util.properties.Properties("RelativeClusterer Properties");

		try{


			Property p = new Property("SimSort", true, "Sort objects in clusters according to similarity to cluster centroid.");
			PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
			p.addValueDescription(pvd);
			pvd = new PropertyValueDescription(false, "Do not do it.");
			p.addValueDescription(pvd);
			properties.addProperty(p);

		}catch(PropertyNameConflictException pnce){
			// Won't happen here
		}


	}


	public IObjectGrouping cluster(IMatrix matrix, IObjectSet ioset){
		
		boolean row=false;
		IObjectSet otherSet = matrix.getColumnIObjectSet();
		if(otherSet==ioset){
			row=true;
			otherSet = matrix.getRowIObjectSet();
		}
		IObjectGrouping otherGrouping = otherSet.getCurrentGrouping();
		
		IObjectGrouping relGrouping = 
			Sorter.makeWeightGroupingRelativeGroupingOfOtherSet(ioset, otherGrouping, matrix);
		
		if((Boolean) properties.getPropertyValue("SimSort"))
			relGrouping.sort();
		
		String name = "Weight Grouping relative ("+otherGrouping.getName()+") "+relativeClusteringNumber;
		relGrouping.setName(name);
		int i=1;
		while(!ioset.setGrouping(relGrouping)){
			relGrouping.setName(name+" ("+i+")");
			i++;
		}

		relativeClusteringNumber++;		
		
		return relGrouping;
	}







	public String getInfoString(){
		return "Makes a clustering relative to the grouping of the other dimension.";
	}

	public String getAlgorithmName(){
		return "RelativeClusterer";    	
	}

	public Vector getPropertyVector(){
		return properties.getPropertyVector();
	}


	public Object getPropertyValue(String name){
		return properties.getPropertyValue(name);
	}
	public void setProperty(String name, Object value) throws PropertyNotFoundException{
		properties.setProperty(name, value);
	}

	public mro.util.properties.Properties getProperties(){
		return properties;
	}
	public void setProperties(mro.util.properties.Properties properties){
		this.properties=properties;
	}

	public UniqueIdentifier getUniqueIdentifier(){
		return identifier;
	}

	public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
		this.identifier=uniqueIdentifier;
	}




}
