package infomat.vectorspace;

import java.io.*;
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving a Corpus. 
 * This class is for my personal use. It does not correspond to anything
 * in this Infomat-package.
 *
 * Not the same as in the clustering package!
 * 
 * @version          061215
 * @author           Magnus Rosell
 */
public class CorpusIO{

	// The InfomatModel that is loaded/saved
	//    private InfomatModel im;

	public CorpusIO(){ //InfomatModel im){
		//	this.im=im;
	}



	/**
	 * Loads a corpus from file in XML format
	 */
	public IMatrix load(File file) {
		try {
			BufferedReader in = 
				new BufferedReader(new FileReader(file));
			InputSource source = new InputSource(in);
			return load(source);
		}
		catch(IOException ioE) {
			System.out.println(ioE);
			ioE.printStackTrace();
		}
		return null;
	}

	public IMatrix loadCompressed(File file) {
		try {
			// Create a ZipInputStream to read the zip file            
			ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(file)));
			InputSource source = new InputSource(zis);
			ZipEntry entry;
			if((entry = zis.getNextEntry()) != null) {
				if(!entry.isDirectory()) {
					return load(source);
				}
			}
			zis.close();
		}
		catch(IOException ioE) {
			System.out.println(ioE);
			ioE.printStackTrace();
		}
		return null;

	}

	private IMatrix load(InputSource source) {
		try {

			IMatrix m = new IMatrix("Words", "Documents");

			//	    im.newIMatrix();

			// Create parser+contentHandler and read the XML document
			XMLReader parser = XMLReaderFactory.createXMLReader();
			CorpusHandler handler = new CorpusHandler(m);
			parser.setContentHandler(handler);
			parser.parse(source);

			return m;

			//	    corpus.setUniqueIdentifier(handler.getUniqueIdentifier());
		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Exports the corpus in XML format
	 */
	/*
    public void save(String fileName) {
        try {
	    File f = new File(fileName);
	    File dir = f.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

            PrintWriter out =
                new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(fileName))));
            save(out);
            out.close();
        }
        catch(IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();
        }
    }
	 */

	/**
	 * Exports the corpus in XML format
	 * The file is saved in compressed .zip-format.
	 */
	/*
    public void saveCompressed(String fileName) {
        try {
            // Create zip compressed output stream, zip file entry and PrintWriter
            ZipOutputStream targetStream = new ZipOutputStream(new FileOutputStream(fileName+".xml.zip"));
            File file = new File(fileName);                        
            targetStream.putNextEntry(new ZipEntry(file.getName() + ".xml"));
            PrintWriter out = new PrintWriter(targetStream);
            save(out);            
            // Close compressed output stream and PrintWriter
            out.flush();
            targetStream.closeEntry();
            targetStream.close();
            targetStream = null;
            out.close();
        }
        catch (java.io.IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();
        }
    }
	 */



	/*
    public void printDTD(PrintWriter out) {
        //We use the xml-format specified in corpus.dtd
        //out.println("<?xml version='1.0' encoding='ISO-8859-1'?>");
        //Bizarr, verkar inte funka att referera till en extern .dtd fil, mste lgga i prologen.
        out.println("<?xml version='1.0' encoding='ISO-8859-1' standalone=\"yes\"?>");
	out.println("<?xml-stylesheet type=\"text/xsl\" href=\""+corpus.sourcePath+"default/Corpus.xsl\"?>");

        //out.println("<!DOCTYPE Corpus SYSTEM \"corpus.dtd\">");
        out.println("<!DOCTYPE Corpus [");

	UniqueIdentifier corpusIdentifier = corpus.getUniqueIdentifier();

        out.println("<!ELEMENT Corpus ("+corpusIdentifier.getDTDElementName()+", CorpusSource, WordImageList,(WordList?),DocumentList)>");

	// UniqueIdentifier
	corpusIdentifier.printPartDTD(out);

 	out.println("<!--The Source of the Corpus-->"); // MR
	out.println("<!ELEMENT CorpusSource (FFPath?, FFFile?, TFPath?, TFFile?)>"); // MR
        out.println("<!ELEMENT FFPath (#PCDATA)> <!--Path of the file file-->"); // MR
        out.println("<!ELEMENT FFFile (#PCDATA)> <!--The file file, containing a list of the files in the corpus-->"); // MR
        out.println("<!ELEMENT TFPath (#PCDATA)> <!--Path of the token file-->"); // MR
        out.println("<!ELEMENT TFFile (#PCDATA)> <!--The token file containg all tokens in the corpus in order per document-->"); // MR

	out.println("<!--The first part contains a WordImgList, i.e. the dimensions of the vector space-->");
        out.println("<!ELEMENT WordImageList (WI+)>");
        out.println("<!ELEMENT WI (ID,STM,F,GW)>"); // MR
        out.println("<!ELEMENT ID (#PCDATA)> <!--wordImageId-->");
        out.println("<!ELEMENT STM (#PCDATA)> <!--stem-->");
        out.println("<!ELEMENT F (#PCDATA)> <!--frequency-->");
        out.println("<!ELEMENT GW (#PCDATA)> <!--Global Weight-->"); // MR

        out.println("<!--The second part contains an optional WordList representing the corpus centroid-->");
        out.println("<!ELEMENT WordList (WD+)>");
        out.println("<!--Words are stored by reference to wordImageId:s-->");
        out.println("<!ELEMENT WD (ID,F?,W, LW?)>");  // MR
        out.println("<!--ID defined above-->");
        out.println("<!--F defined above-->");
        out.println("<!ELEMENT W (#PCDATA)> <!--weight-->");
        out.println("<!ELEMENT LW (#PCDATA)> <!--Local Weight-->");  // MR

        out.println("<!--The third part contains the documents of the corpus-->");
        out.println("<!ELEMENT Documentist (DOC+)>");
        out.println("<!ELEMENT DOC (DOCID,FILE,TITLE?,WordList)>");
        out.println("<!ELEMENT DOCID (#PCDATA)> <!--document id-->");
        out.println("<!ELEMENT FILE (FP,FN)> <!--document file-->");
        out.println("<!ELEMENT FP (#PCDATA)> <!--document file path-->");
        out.println("<!ELEMENT FN (#PCDATA)> <!--document file name-->");
        out.println("<!ELEMENT TITLE (#PCDATA)> <!--document title-->");
        out.println("<!--WordList defined above-->");
        out.println("]>");
    }
	 */

	/*
    private void save(PrintWriter out) {

        printDTD(out);

        out.println("<Corpus>");

	// <UniqueIdentifier>
	corpus.getUniqueIdentifier().printPartXML(out);

	out.println("\t<CourpusSource>");
	out.println("\t\t<FFPath>"+corpus.fileFilePath+"</FFPath>");
	out.println("\t\t<FFFile>"+corpus.fileFile+"</FFFile>");
	out.println("\t\t<TFPath>"+corpus.tokenFilePath+"</TFPath>");
	out.println("\t\t<TFFile>"+corpus.tokenFile+"</TFFile>");
	out.println("\t</CourpusSource>");

        //First we write the WordImageList
        out.println("\t<WordImageList>");
        //Iterate over the Vector wordImageList in WordImageList
        //for(WordImage wi : wordImageList.wordImageList) { //funkar inte eftersom typinfo saknas
        ListIterator wordImageIterator = corpus.wordImageList.wordImageList.listIterator();
        while(wordImageIterator.hasNext()) {
            WordImage wi = (WordImage)wordImageIterator.next();            
            out.println("\t\t<WI>");
            out.println("\t\t\t<ID>" + wi.wordImageId + "</ID>");
            out.println("\t\t\t<STM>" + wi.wordImage + "</STM>");
            out.println("\t\t\t<F>" + wi.freq + "</F>");
            out.println("\t\t\t<GW>" + wi.globalWeight + "</GW>");  // MR
            //Here we should save the wordFormList, it is however optional
            out.println("\t\t</WI>");
        }
        out.println("\t</WordImageList>");
        out.println();

        //Secondly, we write the corpus centroid (if exportCentroid == true)
        if(exportCentroid) {
            out.println("\t<WordList>");
            //Iterate over the Vector wordList in WordList superclass to DocumentGroup superclass to Corpus
            ListIterator wordIterator = corpus.wordList.listIterator();
            while(wordIterator.hasNext()) {
                Word word = (Word)wordIterator.next();                
                out.println("\t\t<WD>");
                out.println("\t\t\t<ID>" + word.wordImage.wordImageId + "</ID>");
                if(exportFreq) out.println("\t\t\t<F>" + word.freq + "</F>");
                out.println("\t\t\t<W>" + word.weight + "</W>"); // MR
		//                if(exportWeight) out.println("\t\t\t<LW>" + word.localWeight + "</LW>");  // MR
                out.println("\t\t</WD>");
            }
            out.println("\t</WordList>");
            out.println();
        }

        //Then we write the actual documents, using references to wordImage ID's
        out.println("\t<DocumentList>");

        //Iterate over the Vector documentGroup in DocumentGroup superclass to Corpus
        //for(Document doc : documentGroup) { //funkar inte eftersom typinformation saknas
        ListIterator documentIterator = corpus.documentGroup.listIterator();        
        while(documentIterator.hasNext()) {
            Document doc = (Document)documentIterator.next();
            out.println("\t\t<DOC>");
            out.println("\t\t\t<DOCID>" + doc.documentId + "</DOCID>");
            out.println("\t\t\t<FILE>");
            out.println("\t\t\t\t<FP>" + doc.getFilePath() + "</FP>");
            out.println("\t\t\t\t<FN>" + doc.getFileName() + "</FN>");
            out.println("\t\t\t</FILE>");
            if(doc.title != null) out.println("\t\t\t<TITLE>" + doc.title + "</TITLE>");
            out.println("\t\t\t<WordList>");
            //Iterate over the Vector wordList in WordList superclass to Document
            //for(Word word : doc.wordList) { //funkar inte eftersom typinfo saknas
            ListIterator wordIterator = doc.wordList.listIterator();            
            while(wordIterator.hasNext()) {
                Word word = (Word)wordIterator.next();
                out.println("\t\t\t\t<WD>");
                out.println("\t\t\t\t\t<ID>" + word.wordImage.wordImageId + "</ID>");
                if(exportFreq) out.println("\t\t\t\t\t<F>" + word.freq + "</F>");
                if(exportWeight) out.println("\t\t\t\t\t<W>" + word.weight + "</W>");
                if(exportWeight) out.println("\t\t\t\t\t<LW>" + word.localWeight + "</LW>");  // MR
                out.println("\t\t\t\t</WD>");
            }
            out.println("\t\t\t</WordList>");
            out.println("\t\t</DOC>");
        }        
        out.println("\t</DocumentList>");
        out.println();
        out.println("</Corpus>");
        out.flush();
    }
	 */

	private class CorpusHandler extends DefaultHandler {
		//        private InfomatModel im;
		private IMatrix m;

		private StringBuffer buffer;

		/*        
        private WordList _wordList = null; //used for Documents as well as the corpus wordlist, first it is used for reading the corpus wordList        
		 */

		private int id;
		private int docId;
		private String stem;
		private float freq;
		private float weight;        
		private float localWeight;   // MR
		private float globalWeight;  // MR

		private String title;
		//private String fileName;

		/*
        private Document doc = null; //used for document specific elements, _wordList points to this when reading documents
        private String fileName;
        private String filePath;
        private String title;

	private String uiOwnerClass;
	private long time;
	private long threadID;

	private UniqueIdentifier ui;
		 */

		public CorpusHandler(IMatrix m){ //InfomatModel im) {
			/*
            corpus.idDocumentMap = new HashMap<Integer,Document>();
            corpus.wordImageList.idWordImageMap = 
		new HashMap<Integer,WordImage>();
            this.corpus = corpus;
            _wordList = corpus;

	    ui = new UniqueIdentifier(corpus);
			 */
			//	    this.im=im;
			this.m=m;
		}

		/*
	public UniqueIdentifier getUniqueIdentifier(){
	    return ui;
	}
		 */

		public void startElement (String namespaceURI, String localName,
				String qualifiedName, Attributes atts) {
			buffer = new StringBuffer();

			/*
            if(qualifiedName.equals("DOC")) {                

                doc = new Document(corpus,-1); //-1 will be changed later when we have read the real id!
                _wordList = doc; //set _wordList to doc since documents are stored using the WordList tag
            }
			 */

			if(qualifiedName.equals("WordList")) {                
				// Mste spara doc innan orden i det.
				//		im.getIMatrix().addDocument(title, docId);
				m.addDocument(title, docId);
			}
		}

		public void endElement (String namespaceURI, String localName, String qualifiedName) {
			if(qualifiedName.equals("STM"))
				stem = buffer.toString();                    
			else if(qualifiedName.equals("ID"))
				id = Integer.parseInt(buffer.toString());
			else if(qualifiedName.equals("WI")) {
				//		im.getIMatrix().addWord(stem, id);
				m.addWord(stem, id);
				//System.out.println(stem);
				/*
                WordImage wordImage = new WordImage(stem,null,freq,false,id);
		wordImage.globalWeight=globalWeight; // MR
                wordImage=corpus.wordImageList.insertWordImage(wordImage);
				 */
			}
			else if(qualifiedName.equals("F"))
				freq = Float.parseFloat(buffer.toString());
			else if(qualifiedName.equals("W"))
				weight = Float.parseFloat(buffer.toString());            
			else if(qualifiedName.equals("LW")) // MR
				localWeight = Float.parseFloat(buffer.toString());            
			else if(qualifiedName.equals("GW")) // MR
				globalWeight = Float.parseFloat(buffer.toString()); 
			else if(qualifiedName.equals("WD")) {
				//		im.getIMatrix().addCell(docId, id, weight);
				m.addCell(docId, id, weight);

				/*
		WordImage wI=corpus.wordImageList.idWordImageMap.get(id);
                Word word = new Word(wI,freq);
		word.localWeight=localWeight; // MR
                word.weight = weight;
                _wordList.insertWord(word);
		wI.addWord(word);
				 */
			}
			else if(qualifiedName.equals("DOCID")){
				docId = Integer.parseInt(buffer.toString());
			}

			/*
            else if(qualifiedName.equals("FP"))
                filePath = buffer.toString();
            else if(qualifiedName.equals("FN"))
		fileName = buffer.toString();
			 */

			else if(qualifiedName.equals("TITLE"))
				title = buffer.toString();
			else if(qualifiedName.equals("DOC")) {    
				/*
		if(Document.documentIdIter<docId)
		    Document.documentIdIter=docId+1;
                doc.documentId = docId;
                doc.title = title;
		doc.fileName = fileName;
		String oldFilePath = corpus.filePathMap.get(filePath);
		if(oldFilePath==null){
		    doc.filePath=filePath;
		    corpus.filePathMap.put(filePath, filePath);
		}else{
		    doc.filePath=oldFilePath;
		}

                //Note that the wordList of doc has already been created when reading the WordList tag, since _wordList pointed to doc then.
		corpus.documentGroup.add(doc);
                corpus.idDocumentMap.put(docId,doc);
				 */
			}  

			/*
	    else if(ui.endElementPartXML(buffer, qualifiedName));
			 */

			/*
	    else if(qualifiedName.equals("FFPath"))
		corpus.fileFilePath=buffer.toString();
	    else if(qualifiedName.equals("FFFile"))
		corpus.fileFile=buffer.toString();
	    else if(qualifiedName.equals("TFPath"))
		corpus.tokenFilePath=buffer.toString();
	    else if(qualifiedName.equals("TFFile"))
		corpus.tokenFile=buffer.toString();
			 */

		}

		public void characters (char ch[], int start, int length) {
			if(buffer != null)
				buffer.append(ch,start,length);
		}      
	}




}
