package infomat.vectorspace;

import infomat.view.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving a DocumentGrouping. 
 * This class is for my personal use. It does not correspond to anything
 * in this Infomat-package.
 *
 * Not tha same as in the clustering package!
 * 
 * @version          061127
 * @author           Magnus Rosell
 */
public class DocumentGroupingIO{

    private static int documentGroupingNumberIter=1;


    // The InfomatModel that is loaded/saved
    private InfomatModel im;

    public DocumentGroupingIO(InfomatModel im){
	this.im=im;
    }


    /**
     * Loads a DocumentGrouping from file in XML format
     */
    public IObjectGrouping load(File file) {
	IObjectGrouping grouping = null;
	try {
            BufferedReader in = new BufferedReader(new FileReader(file));
            InputSource source = new InputSource(in);
            grouping = load(source);
        }
        catch(IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();
        }
	return grouping;
    }

    public IObjectGrouping loadCompressed(File file) { 
	IObjectGrouping grouping = null;
        try {
            // Create a ZipInputStream to read the zip file            
            ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(file)));
            InputSource source = new InputSource(zis);
            ZipEntry entry;
            if((entry = zis.getNextEntry()) != null) {
                if(!entry.isDirectory()) {
		    grouping = load(source);
                }
            }
            zis.close();
        }
        catch(IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();
        }
	return grouping;
    }
         

    private IObjectGrouping load(InputSource source) {
	IObjectGrouping grouping = null;
        try {
	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
            DocumentGroupingHandler handler = new DocumentGroupingHandler(im);
	    parser.setContentHandler(handler);
            parser.parse(source);
	    grouping = handler.getGrouping();
        } catch (Exception e) {
            System.err.println(e);
            e.printStackTrace();
        }
	return grouping;
    }
    
    /**
     * Exports the document grouping in XML format
     */
    /*
    public void save(String fileName){
        try {
            PrintWriter out =
                new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(fileName + ".xml"))));
            dgIO.save(out);
            out.close();
        }
        catch(IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();            
        }
    }
    */

    /**
     * Exports the document grouping in XML format
     * The file is saved in compressed .zip-format.
     */
    /*
    public void saveCompressed(DocumentGrouping documentGrouping, String fileName) {
	DocumentGroupingIO dgIO = new DocumentGroupingIO(documentGrouping);
        try {
            // Create zip compressed output stream, zip file entry and PrintWriter
            ZipOutputStream targetStream = new ZipOutputStream(new FileOutputStream(fileName+".xml.zip"));
            File file = new File(fileName);                        
            targetStream.putNextEntry(new ZipEntry(file.getName() + ".xml"));
            PrintWriter out = new PrintWriter(targetStream);
            dgIO.save(out);        
            // Close compressed output stream and PrintWriter
            out.flush(); //We need to flush the PW before closing the ZipEntry
            targetStream.closeEntry();
            targetStream.close();
            targetStream = null;
            out.close();
        }
        catch (java.io.IOException ioE) {
            System.out.println(ioE);
            ioE.printStackTrace();            
        }
    }
    */

    /*
    private void writeDTD(PrintWriter out){
        //We use the xml-format specified in document_grouping.dtd
        out.println("<?xml version='1.0' encoding='ISO-8859-1' standalone=\"yes\"?>");

	out.println("<?xml-stylesheet type=\"text/xsl\" href=\""+corpus.sourcePath+"default/DocumentGrouping.xsl\"?>");

        //funkar inte att använda extern DTD, måste ha den i prologen (konstigt, verkar bero på SAX-parsern)
        //out.println("<!DOCTYPE DocumentGrouping SYSTEM 'document_grouping.dtd'>");
        out.println("<!DOCTYPE DocumentGrouping [");
        out.println("<!ELEMENT DocumentGrouping (DocumentGroupingType, Identifiers, ConstructionMethod?, DocumentGroupList)>");

        out.println("<!ELEMENT DocumentGroupingType (#PCDATA)> <!--Clustering, or Categorization, ...-->");

	out.println("<!ELEMENT Identifiers (DocumentGroupingIdentifier, CorpusIdentifier, ConstuctionMethodIdentifier)>");
        out.println("<!ELEMENT DocumentGroupingIdentifier (UniqueIdentifier)> <!--Unique identifier for this DocumentGrouping-->");
        out.println("<!ELEMENT CorpusIdentifier (UniqueIdentifier)> <!--Which Corpus is grouped?-->");
        out.println("<!ELEMENT ConstructionMethodIdentifier (UniqueIdentifier)> <!--How this grouping was constructed-->");

	UniqueIdentifier.staticPrintPartDTD(out);

	// Fljande tas bort
	//        out.println("<!ELEMENT ConstructionMethod (#PCDATA)> <!-- How it was created-->"); 

        out.println("<!ELEMENT DocumentGroupList (DocumentGroup+)> <!-- Several DocumentGroup:s -->"); 
	out.println("<!--A group of documents-->");
        out.println("<!ELEMENT DocumentGroup (DGID,Name?,DGD?,WordList?,Documents)>");
        out.println("<!ELEMENT DGID (#PCDATA)> <!--documentGroupId-->");
        out.println("<!ELEMENT Name (#PCDATA)> <!--Name of the document group-->");
        out.println("<!ELEMENT DGD (#PCDATA)> <!--Description of the document group-->");
        out.println();
        out.println("<!--The first part contains an optional WordList representing the document group's centroid-->");
        out.println("<!ELEMENT WordList (WD+)>");
        out.println("<!ELEMENT WD (ID,F?,W, LW?)>"); // MR
        out.println("<!ELEMENT ID (#PCDATA)> <!--wordImage identifier-->");
        out.println("<!ELEMENT F (#PCDATA)> <!--term frequency-->");
        out.println("<!ELEMENT W (#PCDATA)> <!--weight-->");
        out.println("<!ELEMENT LW (#PCDATA)> <!--Local Weight-->"); // MR
        out.println();
        out.println();
        out.println("<!--The second part contains a Document Group, i.e. a list of references to document ID's-->");
        out.println("<!ELEMENT Documents (D+)>");
        out.println("<!ELEMENT D (DID,DS?)>");
        out.println("<!ELEMENT DID (#PCDATA)> <!--documentID-->");
        out.println("<!ELEMENT DS (#PCDATA)> <!--similarity between the document and the document group-->");
        out.println();
        out.println("]>");
    }
    */

    /*
    private void save(PrintWriter out) {        

	writeDTD(out);

	out.println("<DocumentGrouping>");

        //THIS IS NOT A GOOD SOLUTION. WE SHOULD SEND THE TYPE AS A PARAMETER AND IMPLEMENT A WRAPPER FUNCTION IN ALL SUBCLASSES TO DOCUMENTGROUPING INSTEAD.
	String type="unknown";
        if(documentGrouping instanceof Clustering)
            type="Clustering";
	//            out.println("<DocumentGrouping type='clustering'>");
        else if(documentGrouping instanceof Categorization)
            type="Categorization";
	//            out.println("<DocumentGrouping type='categorization'>");
        out.println("\t<DocumentGroupingType>"+type+"</DocumentGroupingType>");
        out.println("<Identifiers>");
        out.println("<DocumentGroupingIdentifier>");
	documentGrouping.getUniqueIdentifier().printPartXML(out);
        Out.println("</DocumentGroupingIdentifier>");

        out.println("<CorpusIdentifier>");
	corpus.getUniqueIdentifier().printPartXML(out);
        out.println("</CorpusIdentifier>");

        out.println("<CostructionMethodIdentifier>");
	documentGrouping.getConstructionMethodIdentifier().printPartXML(out);
        out.println("</CostructionMethodIdentifier>");
        out.println("</Identifiers>");


//        out.println("\t<DocumentGroupingIdentifier>"); // MR
//	UniqueIdentifier ui=documentGrouping.getUniqueIdentifier();
//	out.println("\t\t<Time>" + ui.getTime() + "</Time>"); // MR
//	out.println("\t\t<Thread>" + ui.getThreadID() + "</Thread>"); // MR
//        out.println("\t</DocumentGroupingIdentifier>"); // MR
	
//        out.println("\t<CorpusIdentifier>"); // MR
//	ui=corpus.getUniqueIdentifier();
//	out.println("\t\t<Time>" + ui.getTime() + "</Time>"); // MR
//	out.println("\t\t<Thread>" + ui.getThreadID() + "</Thread>"); // MR
//        out.println("\t</CorpusIdentifier>"); // MR

//	if(documentGrouping.getConstructionMethodString()!=null){
//	    out.print("\t<ConstructionMethod>"); // MR
//	    out.print(documentGrouping.getConstructionMethodString()); // MR
//	    out.println("</ConstructionMethod>"); // MR
//	}


        out.println("<DocumentGroupList>");
        //Iterate over Clusters in Vector documentGroupList in superclass DocumentGrouping
        ListIterator documentGroupIterator = documentGrouping.documentGroupList.listIterator();
        while(documentGroupIterator.hasNext()) {
            DocumentGroup documentGroup = (DocumentGroup)documentGroupIterator.next();
            out.println("\t<DocumentGroup>");            
            out.println("\t\t<DGID>"+documentGroup.getId()+"</DGID>");
            if(documentGroup.getName() != null && documentGroup.getName().length() != 0)
                out.println("\t\t<Name>"+documentGroup.getName()+"</Name>");
            if(documentGroup.getDescription() != null && documentGroup.getDescription().length() != 0)
                out.println("\t\t<DGD>"+documentGroup.getDescription()+"</DGD>");
	    
            if(exportCentroids) {
                out.println("\t\t<WordList>");
		documentGroup.sortByWeight();
                //Iterate over Vector wordListSortedByWeight in WordList superclass to DocumentGroup superclass to Cluster
                ListIterator wordIterator = documentGroup.wordListSortedByWeight.listIterator();
                while(wordIterator.hasNext()) {
                    Word word = (Word)wordIterator.next();
                    out.println("\t\t\t<WD>");
                    out.println("\t\t\t\t<ID>" + word.wordImage.wordImageId + "</ID>");
                    if(exportFreq) out.println("\t\t\t\t<F>" + word.freq + "</F>");
                    out.println("\t\t\t\t<W>" + word.weight + "</W>"); // MR
		    //                    if(exportWeight) out.println("\t\t\t\t<LW>" + word.weight + "</LW>"); // MR
                     out.println("\t\t\t</WD>");
                }            
                out.println("\t\t</WordList>");
                out.println();
            }

            out.println("\t\t<Documents>");

            //Iterate over documents in Cluster            
	    if(exportDocumentClusterSimilarity){
		documentGroup.sortDocumentsByValue();
	    }
            ListIterator documentIterator = documentGroup.documentGroup.listIterator();

            while(documentIterator.hasNext()) {
                Document doc = (Document)documentIterator.next();
                out.print("\t\t\t<D><DID>" + doc.documentId + "</DID>");
		if(exportDocumentClusterSimilarity){
		    boolean normalize=false;
		    out.print("<DS>"+doc.similarity(documentGroup, normalize)+
			      "</DS>");
		    out.println("</D>");
		}else
		    out.println("</D>");
            }
            out.println("\t\t</Documents>");
            out.println("\t</DocumentGroup>");
        }
        out.println("</DocumentGroupList>");
        out.println("</DocumentGrouping>");
    }
*/

    /**
     * Nested class to handle XML parsing.
     */
    private class DocumentGroupingHandler extends DefaultHandler {

	private InfomatModel im;
	//	private Vector<Integer> docsInGroup;
	/*
	private Corpus corpus;
        private DocumentGrouping documentGrouping;
        private DocumentGroup documentGroup;
	*/

	//private String documentGroupingName;
	//private int documentGroupingID;

        private String documentGroupName;
        private String type=null;
	
        private int documentGroupId;
        private int id;
        private float freq;
        private float weight;
        private float localWeight;
	
        private int documentId;

	private float docSim=-1.0f;



	/*
	private String uiOwnerClass;
	private long time;
	private long threadID;
        */

        private StringBuffer buffer;        

	/*
	UniqueIdentifier documentGroupingIdentifier;
	boolean dgI=false;

	UniqueIdentifier corpusIdentifier;
	boolean cI=false;

	UniqueIdentifier constructionMethodIdentifier;
	boolean cmI=false;
        */

	private IObjectGrouping grouping=null;
	private IObjectGroup group;
	private Vector<IObject> cobjects;
	private Vector<IObjectGroup> cObjectGroups;

	public IObjectGrouping getGrouping(){
	    return grouping;
	}


	public DocumentGroupingHandler(InfomatModel im){
	    this.im=im;

	    /*
	    this.corpus=corpus;
	    documentGroupingIdentifier = new UniqueIdentifier();
	    corpusIdentifier = new UniqueIdentifier();
	    constructionMethodIdentifier = new UniqueIdentifier();
	    */
	}

        public void startElement (String namespaceURI, String localName,
                                  String qualifiedName, Attributes atts) {
            buffer = new StringBuffer();

	    if(localName.equals("DocumentGrouping")) {
		cObjectGroups = new Vector<IObjectGroup>();
	    }
	    else if(localName.equals("DocumentGroup")) {
		cobjects = new Vector<IObject>();

		/*
		if(type.equals("Categorization"))
		    docsInGroup=new Vector<Integer>();
		*/

		/*
                if(type.equals("Clustering"))
                    documentGroup = new Cluster(corpus);
                else if(type.equals("Categorization"))
                    documentGroup = new Category(corpus);
                else 
                    documentGroup = new DocumentGroup(corpus);
		*/
            }
            else if(qualifiedName.equals("Documents")) {
		group = new IObjectGroup(documentGroupName, documentGroupId); //, im.getIMatrix());
		cObjectGroups.add(group);
	    }

	    /*
	    else if(localName.equals("DocumentGroupingIdentifier"))
		dgI=true;
	    else if(localName.equals("CorpusIdentifier"))
		cI=true;
	    else if(localName.equals("ConstructionMethodIdentifier"))
		cmI=true;
	    */
        }
        
        public void endElement (String namespaceURI, String localName, String qualifiedName) {


	    if(localName.equals("DocumentGrouping")) {
		String groupingName = 
		    "Document Grouping "+documentGroupingNumberIter;
		if(type!=null)
		    groupingName=type+" "+documentGroupingNumberIter;

		grouping = 
		    new IObjectGrouping(groupingName,
					documentGroupingNumberIter);
		documentGroupingNumberIter++;
		grouping.addAllGroups(cObjectGroups);
	    }
            else if(qualifiedName.equals("DocumentGroup")) {
		/*
		group = new IObjectGroup(documentGroupName, documentGroupId, im.getIMatrix());
		group.addAll(cobjects);
		cObjectGroups.add(group);
		*/
	    }
            else if(qualifiedName.equals("DocumentGroupingType")) {
		type=buffer.toString();
	    }	    

	    else if(qualifiedName.equals("DID")){
                documentId = Integer.parseInt(buffer.toString());

		/*
		IObject co = im.getIMatrix().getRowIObjectWithIndex(documentId);
		cobjects.add(co);
		*/


		/*
		if(type.equals("Categorization")){
		    docsInGroup.add(documentId);
		}else{
		*/
		//		    im.getIMatrix().setRowCluster(documentId, documentGroupId);
		    //		    System.out.println("DOC " + documentId + " => group " + documentGroupId);
		    //		}
		

	    }

		
	    /*
            else if(qualifiedName.equals("ConstructionMethod")) {
		documentGrouping.setConstructionMethodString(buffer.toString());
	    }
	    else if(localName.equals("DocumentGroupingIdentifier"))
		dgI=false;
	    else if(localName.equals("CorpusIdentifier"))
		cI=false;
	    else if(localName.equals("ConstructionMethodIdentifier"))
		cmI=false;
	    */

	    /*
	    else if(dgI && documentGroupingIdentifier.endElementPartXML(buffer, qualifiedName));
	    else if(cI && corpusIdentifier.endElementPartXML(buffer, qualifiedName));
	    else if(cmI && constructionMethodIdentifier.endElementPartXML(buffer, qualifiedName));
	    */


            else if(qualifiedName.equals("DGID")) {
                documentGroupId = Integer.parseInt(buffer.toString());
		/*
                //hmm, how should we handle dgId vs. clusterId and categoryId?
                documentGroup.setId(documentGroupId);
		*/
            }
            else if(qualifiedName.equals("Name")) {
                documentGroupName = buffer.toString();
		/*
                //hmm, how should we handle documengGroupName vs. clusterName and CategoryName?
                documentGroup.setName(documentGroupName);
		*/
            }
            else if(qualifiedName.equals("DGD")) {
		//                documentGroup.setDescription(buffer.toString());
            }
            else if(qualifiedName.equals("ID")) {
                id = Integer.parseInt(buffer.toString());
            }
            else if(qualifiedName.equals("F")) {
                freq = Float.parseFloat(buffer.toString());
            }
            else if(qualifiedName.equals("W")) {
                weight = Float.parseFloat(buffer.toString());
            }
            else if(qualifiedName.equals("LW")) { // MR
                localWeight = Float.parseFloat(buffer.toString());
            }
            else if(qualifiedName.equals("WD")) {
		/*
                //note that the wordImageList.idWordImageMap needs to be initialised manually
                //e.g. when reading in the corpus. This is a temporary solution...
                //hmm, what happens if freq or weight isn't set (They're optional according to the dtd)
                Word word = new Word(corpus.wordImageList.idWordImageMap.get(id),freq);
                word.weight = weight;
                word.localWeight = localWeight; // MR
                documentGroup.insertWord(word);
		*/
            }
            else if(qualifiedName.equals("DS")) {
                docSim = Float.parseFloat(buffer.toString());
            }
            else if(qualifiedName.equals("D")) {
		IObject co = 
		    im.getIMatrix().getRowIObjectSet().getIObjectWithId(documentId);
		if(docSim!=-1)
		    group.add(co, docSim);
		else
		    group.add(co);
		docSim=-1.0f;
            }

	    /*
	    else if(qualifiedName.equals("Time")){ // MR
		time = Long.parseLong(buffer.toString());

	    }
            else if(qualifiedName.equals("Thread")) // MR
                threadID = Long.parseLong(buffer.toString());
            else if(qualifiedName.equals("OwnerClassName")) // MR
                uiOwnerClass = buffer.toString();
            else if(qualifiedName.equals("CorpusIdentifier")){ // MR
		// Not used yet.
		System.out.println("  time: "+time);
		System.out.println("  threadID: "+threadID);
		System.out.println("  ownerClassName: "+uiOwnerClass);
	    }
	    */
        }

        public void characters (char ch[], int start, int length) {
            if(buffer != null)
                buffer.append(ch,start,length);
        }      
    }

}
