package infomat.vectorspace;

//import infomat.picture.*;
import infomat.io.*;
import mro.util.*;

import java.io.*;
import java.util.*;
import java.awt.Dimension;

import mro.util.DoubleHashMap;

/**
 * IMatrix represents the relations between two IObjectSet:s.
 * The IMatrixCell class represents the elements of this IMatrix. 
 * This is implemented considering sparse matrices. The IMatrixCells
 * are stored in a DoubleHashMap and are the non zero matrix elements.
 *
 * @param rows A IObjectSet
 * @param cols A IObjectSet
 * @param cellMap A DoubleHashMap that stores the non zero elements of the matrix.
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IMatrix
{
	private IObjectSet rows;
	private IObjectSet cols;
	private DoubleHashMap cellMap;

	static private float maxW; // XXX ugly, warning - this should be fixed


	public IMatrix(String columnLabel, String rowLabel){
		rows = new IObjectSet(rowLabel); 
		cols = new IObjectSet(columnLabel); 
		cellMap = new DoubleHashMap();

		maxW = 0;
	}


	/**
	 * adds a Word as Column to the matrix
	 * @param w  The Word as a string
	 * @param ws  A unique identifier of the word
	 */
	public void addWord(String w, int ws)
	{
		IObject cob = new IObject(w, ws); 
		cols.add(cob);
	}


	/**
	 * removes columns from the matrix
	 * @param xFrom  Lower bound of columns to be removed
	 * @param xTo  Upper bound of columns to be removed
	 */
	/*
    public void removeColumns(int colFrom, int colTo){

	System.out.println("Antal kolumner: "+cols.numberOfIObjects());

	Vector<IObject> removed = cols.remove(colFrom, colTo);
	for(int i=0; i<removed.size(); i++)
	    cellMap.removeAllSecond(removed.elementAt(i));
	System.out.println("Antal kolumner: "+cols.numberOfIObjects());

	resort();
    }
	 */

	/**
	 * removes rows from the matrix
	 * @param yFrom  Lower bound of rows to be removed
	 * @param yTo  Upper bound of rows to be removed
	 */
	/*
    public void removeRows(int rowFrom, int rowTo){
	Vector<IObject> removed = rows.remove(rowFrom, rowTo);
	for(int i=0; i<removed.size(); i++)
	    cellMap.removeAllFirst(removed.elementAt(i));
	resort();
    }
	 */



	/**
	 * adds a Document as a Row to the matrix
	 * @param d  The source of the document
	 * @param di  A unique identifier of the document
	 */
	public void addDocument(String d, int di)
	{
		IObject cob = new IObject(d, di); //, this); //4 - Document
		rows.add(cob);
	}

	/**
	 * Add a IMatrixCell.
	 * @param mc the Matrix cell
	 * @return true if the corresponding IObjects exist in the IObjectSet:s, false otherwise.
	 */
	private boolean addCell(IMatrixCell mc){
		IObject rowObject = 
			rows.getIObjectWithId(mc.getRowIObject().getID());
		IObject colObject = 
			cols.getIObjectWithId(mc.getColumnIObject().getID());
		if(rowObject==null || colObject==null)
			return false;
		cellMap.put(rowObject, colObject, mc);
		return true;
	}
	
	/**
	 * Add the value of a column in a row
	 * @param ri  The unique identifier of the row
	 * @param ci  The unique identifier of the column
	 * @param weight  The value of the column in the row
	 * @return true if row and column exists, false otherwise
	 */
	public boolean addCell(int ri, int ci, float weight)
	{
		IObject rowObject = rows.getIObjectWithId(ri);
		IObject colObject = cols.getIObjectWithId(ci);
		if(rowObject==null || colObject==null)
			return false;
		IMatrixCell cell = 
			new IMatrixCell(rowObject, colObject, weight);
		if(!addCell(cell))
			return false;
		//cellMap.put(rowObject, colObject, cell);

		maxW = Math.max(maxW,weight);
		return true;
	}

	/**
	 * Add the value of a column in a row
	 * @param ri  The unique identifier of the row
	 * @param ci  The unique identifier of the column
	 * @param w  The weight of the column in the row
	 */
	public void addCell(int ri, int ci, double w)
	{
		addCell(ri, ci, (float)w);
	}

	/**
	 * Removes all IObjects that's not in the current IObjectGroupings
	 * from the sets and all their IObjectGroupings. Also removes
	 * all corresponding IMatrixCells.
	 */
	public void purgeMatrixToCurrentGroupings(){
		// Remove IObjects from Sets
		if(rows!=null)
			rows.purgeSetToCurrentGrouping();
		if(cols!=null)
			cols.purgeSetToCurrentGrouping();

		// Remove IMatrix cells that contains IObject:s 
		// that were removed.
		HashSet<IObject> rowSet = new HashSet<IObject>(rows.getOrdered());
		HashSet<IObject> colSet = new HashSet<IObject>(cols.getOrdered());
		Vector cells = getAllCells();
		for(int i=0; i<cells.size(); i++){
			IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
			if(!rowSet.contains(mc.getRowIObject()) ||
					!colSet.contains(mc.getColumnIObject()))
				removeCell(mc);
		}
	}

	public void invertMatrix(){
		// Invert IObjectSet:s
		IObjectSet tmp = rows;
		rows=cols;
		cols=tmp;
		
		// Invert all IMatrixCell:s
		Vector cells = getAllCells();
		cellMap = new DoubleHashMap();
		for(int i=0; i<cells.size(); i++){
			IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
			mc.invert();
			addCell(mc);
			/*
			IObject tmpO = mc.getRowIObject();
			mc.setRowIObject(mc.getColumnIObject());
			mc.setColumnIObject(tmpO);
			*/
		}
	}
	
	
	/**
	 * Returns the "row" IObjectSet.
	 */
	public IObjectSet getRowIObjectSet(){
		return rows;
	}

	/**
	 * Returns the "column" IObjectSet.
	 */
	public IObjectSet getColumnIObjectSet(){
		return cols;
	}

	/**
	 * Returns the current IObjectGrouping for the rows.
	 */
	public IObjectGrouping getCurrentRowGrouping(){
		return rows.getCurrentGrouping();
	}

	/**
	 * Returns the current IObjectGrouping for the columns.
	 */
	public IObjectGrouping getCurrentColumnGrouping(){
		return cols.getCurrentGrouping();
	}

	/**
	 * Returns the DoubleHashMap containing all 
	 * non zero elements of the IMatrix.
	 */
	public DoubleHashMap getCellMap(){
		return cellMap;
	}

	/**
	 * Returns the DoubleHashMap containing all 
	 * non zero elements of the IMatrix.
	 */
	public Vector getAllCells(){
		return cellMap.values();
	}

	/**
	 * Returns the IMatrixCell for the IObject "row" in the "row" IObjectSet
	 * and the IObject "col" in the "column" IObjectSet
	 */
	public IMatrixCell getCell(IObject row, IObject col){
		return (IMatrixCell) cellMap.get(row, col);
	}

	/**
	 * Removes a IMatrixCell from the IMatrix. Does not remove the 
	 * corresponding IObjects from the IObjectSet:s.
	 * @param mc An IMatrixCell
	 */
	public void removeCell(IMatrixCell mc){
		cellMap.remove(mc.getRowIObject(), mc.getColumnIObject());
	}
	
	/**
	 * Revmove all IMatrixCell from the IMatrix. Does not remove the 
	 * corresponding IObjects from the IObjectSet:s.
	 */
	public void removeAllCells(){
		cellMap=new DoubleHashMap();
	}
	
	/**
	 * Returns a Dimension with x-size as the number of "columns"
	 * and y-size as the number of "rows" of the IMatrix.
	 */
	public Dimension getSize(){
		return new Dimension(cols.numberOfIObjects(), rows.numberOfIObjects());
	}

	/**
	 * Returns a Dimension with x-size as the number of "columns"
	 * and y-size as the number of "rows" of the IMatrix.
	 * This is the size of the matrix seen through the current row and
	 * column groupings.
	 */
	public Dimension getCurrentSize(){
		return new Dimension(getCurrentColumnGrouping().numberOfIObjects(), 
				getCurrentRowGrouping().numberOfIObjects());
	}

	/**
	 * Resorts the IObjectSet:s "rows" and "cols" according to their
	 * current IObjectGroupings.
	 */
	public void resort(){
		//rows.sortAccordingToGrouping(rows.getCurrentGrouping());
		//cols.sortAccordingToGrouping(cols.getCurrentGrouping());
		rows.getCurrentGrouping().sort();
		cols.getCurrentGrouping().sort();
	}

	/**
	 * When all information has been put into the matrix
	 * it needs to be sorted.
	 */
	/*
    public void sort() {

	System.out.println("---------- sort()");

	IObjectGrouping rowGrouping = rows.getCurrentGrouping();
	System.out.println("Radgrupperingens storlek: "+rowGrouping.getGroups().size());
	//	if(rowGrouping!=null)
	//rowGrouping.sortAllGroupsInOrderValueOrder();
	System.out.println("sorterat grupper i vrdeordning");
	//rows.sortAccordingToGrouping(rows.getCurrentGrouping());
	rowGrouping.sort();
	System.out.println("sorterat setet enligt grupperingen");


	IObjectGrouping colGrouping = 
		Sorter.makeWeightGroupingRelativeGroupingOfOtherSet(cols, rowGrouping, this);
	System.out.println("Kolumngrupperingens storlek: "+colGrouping.getGroups().size());
	String name=colGrouping.getName();
	int i=1;
	while(!cols.setGrouping(colGrouping)){
		colGrouping.setName(name+" ("+i+")");
		i++;
	}
	colGrouping.sort();

	//cols.sortAccordingToGrouping(cols.getCurrentGrouping());
    }
	 */

	/**
	 * Returns a Vector of the IMatrixCells that contains the given IObject.
	 */
	Vector getCellsWith(IObject co){
		if(rows.contains(co))
			return cellMap.valuesForFirstIndex(co);
		else if(cols.contains(co))
			return cellMap.valuesForSecondIndex(co);
		return null;
	}

	public HashSet getRowIMatrixCellsForColumnObject(IObject colObject){
		return new HashSet(cellMap.valuesForSecondIndex(colObject));
	}
	
	public HashSet getColumnIMatrixCellsForRowObject(IObject rowObject){
		Collection cells = cellMap.valuesForFirstIndex(rowObject);
		if(cells==null)
			return null;
		return new HashSet(cells);
	}
	
	public void saveIMatrix(File file){
		IMatrixIO iomatrix = new IMatrixIO();
		iomatrix.save(this, file);
	}
	
	public void saveIObjectGroupingFile(IObjectGrouping grouping, File file){
		IObjectGroupingIO iogrouping = new IObjectGroupingIO();
		iogrouping.save(grouping, this, file);
	}

	
	
	
}
