package infomat.vectorspace;

/**
 * An element in a IMatrix. In many types of applications, as Information
 * Retrieval (IR) for instance, the IMatrix is very sparse. Only the non zero
 * elements are stored. 
 *
 * @param rowObject The "row" IObject of the IMatrixCell.
 * @param columnObject The "column" IObject of the IMatrixCell.
 * @param localCount The number of cooccurences of the "row" and "column" IObject.
 * @param localWeight The local weight of this element/IMatrixCell. It may be calculated in many ways. In IR it is often the term frequency. Here it may be the localCount divided by the sum of all localCounts on the "row" or the "column" of the matrix, depending on which is considered the objects and which the features.
 * @param weight The whole weight of this IMatrixCell. Often the localWeight times the global weight.
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IMatrixCell{
	private IObject rowObject;
	private IObject columnObject;

	private int localCount;
	private float localWeight;
	private float weight;

	public IMatrixCell(IObject rowObject, IObject columnObject, 
			int localCount, float weight){
		this.rowObject=rowObject;
		this.columnObject=columnObject;
		this.weight=weight;
		this.localCount=localCount;
	}
	
	public IMatrixCell(IObject rowObject, IObject columnObject, float weight){
		this.rowObject=rowObject;
		this.columnObject=columnObject;
		this.weight=weight;
		this.localCount=-1;
	}

	public float getLocalCount(){
		return localCount;
	}

	public float getLocalWeight(){
		return localWeight;
	}

	public float getWeight(){
		return weight;
	}

	public IObject getRowIObject(){
		return rowObject;
	}

	public IObject getColumnIObject(){
		return columnObject;
	}

	protected void invert(){
		IObject tmp = rowObject;
		rowObject = columnObject;
		columnObject = tmp;
	}
	
	protected void setRowIObject(IObject io){
		rowObject = io;
	}
	
	protected void setColumnIObject(IObject io){
		columnObject = io;
	}
	
}

