package infomat.vectorspace;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

/**
 * A IObject is either an object or a feature.
 * It could be a word or a document for instance.
 * This is the smallest part of the representation.
 *
 * @param s  A String describing the IObject
 * @param kind  The kind of IObject. 0 - IObject, 1 - IObjectGroup, 2 - IObjectGrouping, 3 - IObjectSet
 * @param id  Index, a unique identifier for the IObject
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IObject
{
	private String s=null;
	private int kind;
	private int id=0;

	public IObject(String s, int id){
		this.s=s;
		this.kind=0;
		this.id=id;
	}

	public IObject(String s, int kind, int id){ //, IMatrix myIMatrix){
		this.s=s;
		this.kind=kind;
		this.id=id;
	}

	public String getString(){
		if(s==null)
			return "id("+id+")";
		return s;
	}

	public void setString(String s){
		this.s=s;
	}

	public int getKind(){
		return kind;
	}

	public int getID(){
		return id;
	}


	/**
	 * Returns the representation in the form of a HashMap, mapping
	 * IObject to a Float value. For a group - the centroid.
	 */
	public HashMap<IObject, Float> getRepresentation(IMatrix myIMatrix){
		HashMap<IObject, Float> centroid = new HashMap<IObject, Float>();

		Vector features = myIMatrix.getCellsWith(this);
		if(features!=null){
			for(int f=0; f<features.size(); f++){
				IMatrixCell sc = (IMatrixCell) features.elementAt(f);
				IObject feature = sc.getRowIObject();
				if(this==feature)
					feature = sc.getColumnIObject();
				centroid.put(feature, sc.getWeight());
			}
		}
		return centroid;
	}



	public int stringCompareTo(IObject co){
		return getString().compareTo(co.getString());
	}
}

