package infomat.view;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
	public class AlgorithmFrame extends JFrame
	implements ActionListener{

		public Dimension ALGORITHMFRAME_PREFERRED_SIZE = 
			new Dimension(300, 500);
		
		private GroupingAlgorithm algorithm;
		private IMatrix matrix;
		private InfomatModel model;
		private InfomatView view;
		
		
		private JButton rowsOrColumns;
		private JButton apply;
		private JButton cancel;
		
		public AlgorithmFrame(GroupingAlgorithm algorithm, IMatrix matrix, InfomatModel model, InfomatView view){
			super(algorithm.getAlgorithmName());
			//super(model.getMainWindow().getMainFrame(), algorithm.getAlgorithmName(), true);
			this.algorithm=algorithm;
			this.matrix=matrix;
			this.model=model;
			this.view=view;
			
			if(matrix==null || view==null || model==null){
				dispose();
				return;
			}
			
			//setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
			
			Container container = getContentPane();
			container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
			
			
			JTextField dummy = new JTextField("Apply");
			JButton dummyButton = new JButton("Cancel");
			int height = (int) dummy.getPreferredSize().getHeight();
			int width = (int) dummyButton.getPreferredSize().getWidth();
			Dimension buttonSize = new Dimension(width, height);
			
			JPanel infoAndButtons = new JPanel();
			infoAndButtons.setLayout(new GridLayout(4,1));
			JLabel description = new JLabel("Description: "+algorithm.getInfoString());
			infoAndButtons.add(description);
			JLabel info = new JLabel("Alter properties below and press \"Apply\"");
			infoAndButtons.add(info);
			
			JPanel dimPanel = new JPanel();
			dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
			JLabel dimInfo = new JLabel("Choose rows or columns: ");
			dimPanel.add(dimInfo);
			rowsOrColumns = new JButton("Rows");
			rowsOrColumns.setPreferredSize(buttonSize);
			rowsOrColumns.addActionListener(this);
			dimPanel.add(rowsOrColumns);
			infoAndButtons.add(dimPanel);
			
			JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			apply = new JButton("Apply");
			apply.setPreferredSize(buttonSize);
			apply.addActionListener(this);
			buttonPanel.add(apply);
			cancel = new JButton("Cancel");
			cancel.setPreferredSize(buttonSize);
			cancel.addActionListener(this);
			buttonPanel.add(cancel);
			infoAndButtons.add(buttonPanel);
			infoAndButtons.setBorder(new TitledBorder(algorithm.getAlgorithmName()));
			container.add(infoAndButtons);
			
			
					
			Properties properties = algorithm.getProperties(); 
			PropertiesPanel pp = new PropertiesPanel(properties);
			JScrollPane sp = new JScrollPane();
			sp.setViewportView(pp);
			sp.setBorder(new TitledBorder("Properties"));
			container.add(sp);
			
			setVisible(true);
			setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);
			//setSize(ALGORITHMFRAME_PREFERRED_SIZE);
			//setMinimumSize(ALGORITHMFRAME_PREFERRED_SIZE);
			pack();
		}
		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				JButton source = (JButton) e.getSource();
				if(source == apply){
					ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());
					
					boolean row=true;
					if(rowsOrColumns.getText().equals("Columns"))
						row=false;
					IObjectSet set = matrix.getColumnIObjectSet();
					if(row)
						set = matrix.getRowIObjectSet();
					
					//System.out.println("row: "+row);
					
					IObjectGrouping clustering = algorithm.cluster(matrix, set);
					clustering.removeEmptyGroups();
					
					view.getGroupingManager(row).putNewGrouping(clustering);
					
					
					view.forceChange();
					view.getOverview().forceChange();

					
					model.setChanged();
					model.notifyObservers();

					pf.dispose();
					dispose();
				}else if(source == cancel){
					dispose();
				}else if(source == rowsOrColumns){
					if(rowsOrColumns.getText().equals("Columns"))
						rowsOrColumns.setText("Rows");
					else if(rowsOrColumns.getText().equals("Rows"))
						rowsOrColumns.setText("Columns");
				}
				
			}
		}
		
	}

	
	
