package infomat.view;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


/**
 * The heart! Keeps track of what the corpus and other stuff contains.
 */
public class InfomatModel extends Observable
	implements HasProperties{
	
	private MainWindow mainWindow;
	private IMatrix matrix;
	private InfomatView view;

	/*
	private final static Color[] farger = { Color.BLUE, Color.CYAN,
		Color.MAGENTA, Color.GREEN, 
		Color.YELLOW, Color.PINK };
	 */
	
	private mro.util.properties.Properties properties;
	private static String defaultSettingsFile = "InfomatProperties";
	private String settingsFile;
	
	private ProgressFrame progressFrame;
	
	public InfomatModel(MainWindow mainWindow){
		this.mainWindow=mainWindow;
		String sep = System.getProperty("file.separator");
		settingsFile = System.getProperty("user.dir")+sep+"settings"+sep+defaultSettingsFile;
		//System.out.println("settingsFile: "+settingsFile);
		properties = mro.util.properties.Properties.load(settingsFile);
		/*
		mro.util.properties.Property p = new 
			mro.util.properties.Property("User dir", System.getProperty("user.dir"), "User directory");
		try{
			properties.addProperty(p);
		}catch(PropertyNameConflictException pnce){
			System.out.println("Bad property name"+p.getName());
			System.out.println("Exiting!");
			System.exit(0);
		}
		*/
		//System.out.println("----------"+properties.getPropertyValue("Icon Path"));
		//System.out.println("----------"+properties.getPropertyValue("Icon Path"));
	}
	
    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public Properties getProperties(){
    	return properties;
    }
    
    public void setProperties(Properties properties){
    	this.properties=properties;
    }
	
	protected void setChanged(){
		super.setChanged();
	}
	
	
	
	/**
	 * Do we have a matrix?
	 * 
	 * @return true if we have a matrix
	 */
	public boolean hasIMatrix() {
		return matrix != null;
	}

	/**
	 * 
	 * @return the current matrix object
	 */
	public IMatrix getIMatrix() {
		return matrix;
	}

	public MainWindow getMainWindow(){
		return mainWindow;
	}
	
	/**
	 * New matrix.
	 */
	/*
    public void newIMatrix(){
	matrix = new IMatrix();
    }
	 */

	/*
	public Color[] getFarger(){
		return farger;
	}
	*/

	/**
	 * Set the view. Must me done!!
	 * 
	 * @param view
	 *            The view.
	 */
	public void setView(InfomatView view) {
		this.view = view;
	}


	public void update(){
		setChanged();
		notifyObservers();
	}




	/**
	 * Opens a Corpus File.
	 */
	public void openCorpusFile(File file){
		
		progressFrame = new ProgressFrame("Loading Corpus File");
		matrix = null;
		view.forceChange();
		setChanged();
		notifyObservers();
		
		CorpusIO cio = new CorpusIO();
		matrix = cio.load(file);

		// Make start groupings
		//IObjectGrouping colGrouping = 
		matrix.getColumnIObjectSet().makeStartGrouping();
		//IObjectGrouping rowGrouping = 
		matrix.getRowIObjectSet().makeStartGrouping();

		progressFrame.dispose();

		// Grouping Settings
		boolean row=true;
		view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
		row=false;
		view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));
		
		view.forceChange();
		//view.getOverview().forceChange();
		setChanged();
		notifyObservers();
	}

	/*
	private IObjectGrouping makeRelativeColumnGrouping(IObjectGrouping rowGrouping){
		
		// Make Column Grouping relative Row Grouping
		IObjectSet cols = matrix.getColumnIObjectSet();
		IObjectGrouping relativeColGrouping = 
			Sorter.makeWeightGroupingRelativeGroupingOfOtherSet(cols, rowGrouping, matrix);
		// Unikt namn
		String name=relativeColGrouping.getName();
		int i=1;
		while(!cols.setGrouping(relativeColGrouping)){
			relativeColGrouping.setName(name+" ("+i+")");
			i++;
		}
		return relativeColGrouping;
	}
	*/

	/**
	 * Opens a Document Grouping File.
	 */
	public void openDocumentGroupingFile(File file){
		DocumentGroupingIO dgio = new DocumentGroupingIO(this);
		IObjectGrouping rowGrouping = dgio.load(file);
		rowGrouping.sort();
		IObjectSet cos = matrix.getRowIObjectSet();
		System.out.println("Lst in!!!");

		//fundamentalIMatrixChanges=true;

		// Just nu stts grupperingen till den man senast lst in!
		String name = rowGrouping.getName();
		int i=1;
		while(!cos.setGrouping(rowGrouping)){
			rowGrouping.setName(name+" ("+i+")");
			i++;
		}
		//	cos.addGrouping(rowGrouping);



		// Grouping Settings
		view.getRowGroupingManager().putNewGrouping(rowGrouping);

		/*
		IObjectGrouping relativeColGrouping = 
			makeRelativeColumnGrouping(rowGrouping);
		view.getColumnGroupingManager().putNewGrouping(relativeColGrouping);
		*/

		view.forceChange();
		setChanged();
		notifyObservers();
	}

	/*
    public boolean inquireFundamentalChanges(){
	boolean ret = fundamentalIMatrixChanges;
	fundamentalIMatrixChanges=false;
	return ret;
    }
	 */


	public void removeColumns(int colFrom, int colTo){
		//matrix.removeColumns(colFrom, colTo);
		matrix.getCurrentColumnGrouping().remove(colFrom, colTo);
	}


	public void removeRows(int rowFrom, int rowTo){
		//matrix.removeRows(colFrom, colTo);
		matrix.getCurrentRowGrouping().remove(rowFrom, rowTo);
	}


	public void resortIMatrix(){
		matrix.resort();
		update();
	}


	/**
	 * Reset settings about zoom and position in the image.
	 * 
	 */
	public void resetImage() {
		view.resetImage();
		update();
	}


	public void saveIMatrixPicture(File file){
		view.saveIMatrixPicture(file);
	}

	public void saveIMatrixFile(File file){
		matrix.saveIMatrix(file);
	}

	public void saveIObjectGroupingFile(IObjectGrouping grouping, File file){
		matrix.saveIObjectGroupingFile(grouping, file);
	}
	
	public void openIMatrixFile(File file){
		
		matrix = null;
		view.forceChange();
		setChanged();
		notifyObservers();
		
		IMatrixIO iomatrix = new IMatrixIO();
		matrix = iomatrix.load(file);

		IObjectGrouping rowGrouping = 
			matrix.getColumnIObjectSet().makeStartGrouping();
		IObjectGrouping colGrouping = 
			matrix.getRowIObjectSet().makeStartGrouping();

		rowGrouping.sort();
		colGrouping.sort();

		// Grouping Settings
		boolean row=true;
		view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
		row=false;
		view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));

		/*
		// Colorings
		view.getRowGroupingManager().makeNewColoringsForAllGroupings();
		view.getColumnGroupingManager().makeNewColoringsForAllGroupings();
		*/
		
		/*
		boolean row=true;
		view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getRowIObjectSet());
		row=false;
		view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getColumnIObjectSet());
		 */

		setChanged();
		notifyObservers();
	}

	public IObjectGrouping openIObjectGroupingFile(boolean row, File file){
		IObjectGroupingIO iogrouping = new IObjectGroupingIO(); 
		IObjectGrouping grouping = iogrouping.load(row, matrix, file);

		if(grouping!=null){
			IObjectSet set = matrix.getColumnIObjectSet();
			if(row)
				set = matrix.getRowIObjectSet();

			String name = grouping.getName();
			int i=1;
			while(!set.addGrouping(grouping)){
				grouping.setName(name+" ("+i+")");
				i++;
			}

			view.getGroupingManager(row).putNewGrouping(grouping);
		}

		return grouping;
	}

	
	
	
	
	public void purgeMatrix(){
		getIMatrix().purgeMatrixToCurrentGroupings();
		setChanged();
		notifyObservers();
	}

	public void invertMatrix(){
		/*
		System.out.println("model.invertMatrix()");
		System.out.println("current matrix size: "+getIMatrix().getCurrentSize());
		System.out.println("current col grouping: "+getIMatrix().getCurrentColumnGrouping().getName());
		System.out.println("current row grouping: "+getIMatrix().getCurrentRowGrouping().getName());
		System.out.println("current row manager: ");
		view.getRowGroupingManager().print();
		System.out.println("current col manager: ");
		view.getColumnGroupingManager().print();
		*/
		// Invert Matrix
		getIMatrix().invertMatrix();

		// Invert View 
		view.invert();
		
		/*
		System.out.println("-------------");
		System.out.println("current matrix size: "+getIMatrix().getCurrentSize());
		System.out.println("current col grouping: "+getIMatrix().getCurrentColumnGrouping().getName());
		System.out.println("current row grouping: "+getIMatrix().getCurrentRowGrouping().getName());
		System.out.println("current row manager: ");
		view.getRowGroupingManager().print();
		System.out.println("current col manager: ");
		view.getColumnGroupingManager().print();
		*/
		
		//view.changePartPictures();
		//view.getOverview().changePartPictures();
		
		//view.getSparsePicture().makeImage();
		//view.getOverview().makeImage();
		
		view.forceChange();
		resetImage();
	}

	
	public void randomClustering(){
		RandomClusterer rc = new RandomClusterer();
		cluster(rc);
	}
	
	public void relativeClustering(){
		RelativeClusterer rc = new RelativeClusterer();
		cluster(rc);
	}
	
	public void kmeans(){
		KMeans kmeans = new KMeans();
		cluster(kmeans);
	}

	private AlgorithmFrame algorithmFrame;
	private void cluster(GroupingAlgorithm algorithm){
		if(algorithmFrame!=null)
			algorithmFrame.dispose();
		algorithmFrame = new AlgorithmFrame(algorithm, matrix, this, view);	
	}

	
	
}
