package infomat.view.grouping;

import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.settings.GroupingSettingsManager;
import infomat.view.grouping.settings.GroupingColoring;
import infomat.view.grouping.settings.GroupingSettingsPanel;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;


/**
 * Displayer of info.
 * @see GroupingPanel
 */
public class GroupingSelectionPanel extends JPanel
implements ActionListener{
	//    implements Observer{

	private static String none="None";

	private InfomatModel model;
	private InfomatView view;
	private IObjectSet ios;
	private boolean colorPanel;
	private boolean rowPanel; // Row or column

	private JComboBox cBox;
	private JButton button;

	private String selected;

	private JFrame settingsFrame = null;

	public GroupingSelectionPanel(InfomatModel model, InfomatView view, 
			boolean rowPanel, boolean colorPanel){
		this.model=model;
		this.view=view;
		this.rowPanel=rowPanel;
		this.colorPanel=colorPanel;
		selected=none;
	}

	public boolean actionLeadToChange(){
		String newSelected = (String) cBox.getSelectedItem();
		System.out.println("newSelected: "+newSelected);
		if(selected != newSelected){
			selected = newSelected;
			return true;
		}
		return false;
	}

	public IObjectGrouping getSelectedGrouping(){
		if(selected.equals(none))
			return null;
		return ios.getGroupingWithName(selected);
	}


	public void paintComponent(Graphics g) {
		super.paintComponent(g);
	}



	public void makeComboBox(IObjectSet ios){
		if(ios==null)
			return;

		this.ios=ios;
		removeAll();
		Vector<IObjectGrouping> groupings = ios.getGroupings();
		if(groupings != null && groupings.size()>0){
			Vector<String> strings = new Vector<String>();

			for(int i=0; i<groupings.size(); i++)
				strings.add(groupings.elementAt(i).getName());
			if(colorPanel){
				strings.insertElementAt(none, 0);
				GroupingSettingsManager cm = view.getGroupingManager(rowPanel);
				GroupingColoring gc = cm.getCurrentColoring();
				if(gc==null)
					selected=none;
				else{
					IObjectGrouping grouping = gc.getIObjectGrouping();
					
					//ios.getGroupingWithName(selected);
					//cm.setColoring(rowPanel, grouping);
					if(selected!=null)
						selected=grouping.getName();
					else
						selected=none;
				}

			}else{
				IObjectGrouping grouping = ios.getCurrentGrouping();
				if(grouping!=null)
					selected = grouping.getName();
				else
					selected=none;
			}

			cBox = new JComboBox(strings);
			cBox.setSelectedItem(selected);
		}

		String ls = "Shown";
		if(colorPanel)
			ls="Color   ";
		JLabel l = new JLabel(ls);
		button = new JButton("S");


		cBox.setPrototypeDisplayValue("Clustering of some things");

		add(l);
		add(cBox);
		add(button);

		cBox.addActionListener(this);
		button.addActionListener(this);


		revalidate();
		repaint();
	}


	public void actionPerformed(ActionEvent e) {
		// ComboBox
		if(e.getSource() instanceof JComboBox){    	    
			if(actionLeadToChange()){
				// ColorPanel
				if(colorPanel){
					//System.out.println("GroupingSelectionPanel");
					//System.out.println("   selected: "+selected);
					/*
		    		IObjectGrouping grouping = 
		    			getSelectedGrouping();
		    		if(grouping==null)
		    		    view.getColoringManager().setColoring(rowPanel, null);
		    		else{
		    		    GroupingColoring coloring = 
		    			new GroupingColoring(grouping);
		    		    view.getColoringManager().setColoring(rowPanel, coloring);
		    		}
					 */
					/*
    	    		if(selected.equals(none))
	    	    		ios.setGrouping(null);
    	    		else
	    	    		ios.setGrouping(ios.getGroupingWithName(selected));
					 */

					view.getGroupingManager(rowPanel).setCurrentColoringByGrouping(ios.getGroupingWithName(selected));
					//view.getColoringManager().setColoring(rowPanel, ios.getGroupingWithName(selected));
					view.forceChange();
					model.update();
					//view.update(null, null);

				}
				// Show Panel
				else{
					ios.setGrouping(ios.getGroupingWithName(selected));
					view.resetImage();
					view.forceChange();
					model.resortIMatrix();
				}
			}
		}
		// Button
		else if(e.getSource() instanceof JButton){
			if(settingsFrame == null || 
					!settingsFrame.isShowing()){

				String frameTitle = "Column";
				if(rowPanel)
					frameTitle = "Row";
				if(colorPanel)
					frameTitle+=" Color";
				else
					frameTitle+=" Show";
				frameTitle += " Settings";

				settingsFrame = 
					new JFrame(frameTitle);
				Container container = 
					settingsFrame.getContentPane();
				GroupingSettingsPanel gsp = 
					new GroupingSettingsPanel(model, view, ios, rowPanel, colorPanel);
				gsp.setLayout(new BoxLayout(gsp, BoxLayout.Y_AXIS));
				container.add(gsp);
				//settingsFrame.setSize(100,100);
				//		settingsFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
				settingsFrame.pack();
				settingsFrame.setVisible(true);
			}else{
				settingsFrame.dispose();
				settingsFrame = null;
			}
		}
	}


	public JComboBox getComboBox(){
		return cBox;
	}

	public JButton getButton(){
		return button;
	}

}


