package infomat.view.grouping.settings;

import infomat.vectorspace.*;

import java.awt.*;
import java.util.*;


/**
 * Colors.
 */
public class GroupingColoring{

	private static Color[] defaultColors = 
	{Color.red, Color.green, Color.blue, Color.yellow,
		Color.magenta, Color.orange, Color.cyan, Color.pink};

	private static Color standardBackground=Color.white;
	private static Color standardForground=Color.black;

	private boolean rowColoring;
	private IObjectGrouping coging;
	private HashMap<IObjectGroup, Color> groupColorMap;
	private HashMap<IObjectGroup, Color> newGroupColorMap;

	public GroupingColoring(IObjectGrouping coging, boolean rowColoring){
		this.coging = coging;
		this.rowColoring=rowColoring;
		resetColors();
	}

	public void resetColors(){
		groupColorMap = new HashMap<IObjectGroup, Color>();
		newGroupColorMap = new HashMap<IObjectGroup, Color>();
		Vector<IObjectGroup> groups = coging.getGroups();
		if(groups.size()==1){
			Color color = standardBackground;
			if(rowColoring)
				color=standardForground;
			groupColorMap.put(groups.elementAt(0), color);
			newGroupColorMap.put(groups.elementAt(0), color);	
		}else{
			int length = defaultColors.length;
			for(int i=0; i<groups.size(); i++){
				groupColorMap.put(groups.elementAt(i), defaultColors[i%length]);
				newGroupColorMap.put(groups.elementAt(i), defaultColors[i%length]);
			}    
		}
	}

	public void recolor(){
		System.out.println("-----------recolor");
		System.out.println("Before: "+groupColorMap.size());
		HashMap<IObjectGroup, Color> colorMap = groupColorMap;
		groupColorMap = new HashMap<IObjectGroup, Color>();
		newGroupColorMap = new HashMap<IObjectGroup, Color>();
		Vector<IObjectGroup> groups = coging.getGroups();
		System.out.println("groups: "+groups.size());		
		for(int i=0; i<groups.size(); i++){
			IObjectGroup group = groups.elementAt(i);
			groupColorMap.put(group, colorMap.get(group));
			newGroupColorMap.put(group, colorMap.get(group));
		}    	    	
		System.out.println("After: "+groupColorMap.size());
		System.out.println("-----------recolor");
	}

	public Color getColorForGroup(IObjectGroup cog){
		return groupColorMap.get(cog);
	}

	public Color getNewColorForGroup(IObjectGroup cog){
		return newGroupColorMap.get(cog);
	}

	public void setNewColorForGroup(Color color, IObjectGroup cog){
		newGroupColorMap.put(cog, color);
	}

	public void changeToNewColors(){
		groupColorMap = new HashMap<IObjectGroup, Color>();
		groupColorMap.putAll(newGroupColorMap);
	}

	public Color getColorForObject(IObject co){
		IObjectGroup cog = coging.inWhichIObjectGroup(co);
		if(cog==null)
			return null;
		return getColorForGroup(cog);
	}

	public IObjectGrouping getIObjectGrouping(){
		return coging;
	}


	public static Color getStandardBackground(){
		return standardBackground;
	}
	public static Color getStandardForground(){
		return standardForground;
	}
}
