package infomat.view.grouping.settings;

import infomat.vectorspace.*;
import infomat.view.*;

import javax.swing.*;
import javax.swing.border.LineBorder;

import java.awt.event.*;
import java.awt.*;
import java.util.*;

/**
 * Displayer of info. 
 *
 */
public class GroupingSettingsPanel extends JPanel 
implements Observer{

	private InfomatModel model;
	private InfomatView view;
	private IObjectSet ios;
	private boolean rowPanel;
	private boolean colorPanel;

	private JTextField groupingNameTextField;

	private GridBagConstraints gbc;

	private static int textHeight = 20;
	private static Dimension panelDimension = new Dimension(300, 500);

	public GroupingSettingsPanel(InfomatModel model, InfomatView view,
			IObjectSet ios, boolean rowPanel, boolean colorPanel){
		//super(new GridLayout(3,1));
		//super(new GridBagLayout());
		//gbc = new GridBagConstraints();
		//gbc.fill = GridBagConstraints.VERTICAL;
		//gbc.anchor = GridBagConstraints.LINE_START;

		//	super(new GridLayout(1,2));
		//	setPreferredSize(new Dimension(800, 450));
		this.model=model;
		this.view=view;
		this.ios=ios;
		this.colorPanel=colorPanel;
		this.rowPanel=rowPanel;
		model.addObserver(this);

		setPreferredSize(panelDimension);

		putAll();
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
	}

	public void putAll(){
		removeAll();
		// Grouping
		IObjectGrouping grouping=ios.getCurrentGrouping();
		if(colorPanel){
			GroupingColoring gc = 
				view.getGroupingManager(rowPanel).getCurrentColoring();
			/*
			GroupingColoring gc = 
				view.getColoringManager().getColoring(rowPanel);
			*/
			if(gc==null)
				grouping=null;
			else
				grouping=gc.getIObjectGrouping();
		}
		if(grouping==null){
			add(new Label("No Grouping..."));
		}
		else{
			GroupingInfo ginfo = new GroupingInfo(grouping, model, ios);
			//ginfo.setLayout(new BoxLayout(ginfo, BoxLayout.Y_AXIS));
			add(ginfo);

			// Groups
			add(new Groups(grouping, rowPanel, colorPanel));

			// Ordering
			Ordering ordering = new Ordering(model, view, grouping);
			add(ordering);

			// Color Change
			if(colorPanel){
				add(new ColorChanger(view, grouping, rowPanel));
			}
			
			// Opacity sliders
			if((rowPanel && !colorPanel) ||
				(!rowPanel && colorPanel)){
				GroupingSettings gs = 
					view.getGroupingManager(rowPanel).getSettingsForGrouping(grouping);
				Dimension d = new Dimension((int) panelDimension.getWidth(), (int)panelDimension.getHeight()/5);
				OpacityPanel op = new OpacityPanel(model, view, gs, d);
				op.setLayout(new BoxLayout(op, BoxLayout.Y_AXIS));
				add(op);
			}
		}

		JButton dummy = new JButton("Hej!");
		Dimension d = new Dimension((int) panelDimension.getWidth(), (int) dummy.getPreferredSize().getHeight());//(int)panelDimension.getHeight()/5);
		GroupingFilePanel gfp = new GroupingFilePanel(model, view, grouping, rowPanel, colorPanel, d);
		add(gfp);
		
		revalidate();
		repaint();
	}


	/** Called by the Observable */
	public void update(Observable o, Object arg) {
		putAll();
		revalidate();
		repaint();
	}

	/***************************************************/
	private class GroupingInfo extends JPanel implements ActionListener {

		private InfomatModel model;
		private IObjectGrouping grouping;
		private JTextField groupingNameTextField;
		private IObjectSet ios;
		
		public GroupingInfo(IObjectGrouping grouping, InfomatModel model,
				IObjectSet ios){
			//super(new BorderLayout());
			super(new GridLayout(2,1));
			this.model=model;
			this.grouping=grouping;
			this.ios=ios;
			
			putAll();
		}
		
		public void putAll(){
			removeAll();
			String rowOrColumn ="columns";
			if(grouping == model.getIMatrix().getCurrentRowGrouping())
				rowOrColumn = "rows";
			//String text = "<html><h3>"+rowOrColumn+" Grouping: <b>"+grouping.getName()+"</h3></b></html>"
			String text = "<html><h2>"+grouping.getName()+"</h2></html>";
			JLabel l = new JLabel(text);
			l.setHorizontalAlignment(SwingConstants.CENTER);
			/*
			Dimension dl = l.getPreferredSize();
			l.setMinimumSize(dl);
			l.setMaximumSize(dl);
			*/
			add(l);

			
			JPanel part = new JPanel();
			part.setLayout(new GridLayout(2,1));
			part.add(new JLabel("Number of "+rowOrColumn+": "+grouping.getGroupingVector().size()));
			JPanel part2 = new JPanel();
			part2.setLayout(new BoxLayout(part2, BoxLayout.X_AXIS));
			part2.add(new JLabel("Change Name:"));
			groupingNameTextField = new JTextField(20);
			groupingNameTextField.setText(grouping.getName());
			groupingNameTextField.addActionListener(this);
			part2.add(groupingNameTextField);   		
			part.add(part2);
			add(part);

			int height = (int) (groupingNameTextField.getPreferredSize().getHeight() +
					l.getPreferredSize().getHeight());
			int width = (int) (getPreferredSize().getWidth());
			//int width = (int) (l.getPreferredSize().getWidth());
			Dimension d = new Dimension(width, height);  		
			setPreferredSize(d);
			setMinimumSize(d);
			setMaximumSize(d);

			revalidate();
			repaint();
		}

		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JTextField){
				JTextField t = (JTextField) e.getSource();
				IObjectGrouping grouping = ios.getCurrentGrouping();
				String oldName = grouping.getName();
				if(t == groupingNameTextField){
					String newName = t.getText();
					Vector<IObjectGrouping> groupings = ios.getGroupings();
					for(int i=0; i<groupings.size(); i++){
						if(newName.equals(groupings.elementAt(i).getName()))
							newName=oldName;
					}
					ios.removeGrouping(grouping);
					grouping.setName(newName);
					ios.setGrouping(grouping);
					
					model.update();
				}
				putAll();
			}
		}







	}


	/***************************************************/
	private class Ordering extends JPanel
	implements ActionListener{

		private InfomatModel model;
		private InfomatView view;
		private IObjectGrouping grouping;

		private JTextField textField;
		private JButton button;

		public Ordering(InfomatModel model, InfomatView view,
				IObjectGrouping grouping){
			this.model=model;
			this.view=view;
			this.grouping=grouping;

			JLabel label = new JLabel("Reorder: ");
			add(label);

			String order = getOrderString();
			textField = new JTextField(order);
			add(textField);

			button = new JButton("Apply");
			button.addActionListener(this);
			add(button);

			/*
	    int width = (int) 
	    	(label.getPreferredSize().getWidth()+
	    		textField.getPreferredSize().getWidth()+
	    		button.getPreferredSize().getWidth());
			 */
			int height = (int) textField.getPreferredSize().getHeight();
			button.setPreferredSize(new Dimension((int)button.getPreferredSize().getWidth(), height));

			Dimension d = getPreferredSize(); //new Dimension(width, height);
			setPreferredSize(d);
			setMinimumSize(d);
			setMaximumSize(d);

			//setBorder(new LineBorder(Color.BLACK));
		}

		private String getOrderString(){
			String order = "1";
			for(int i=1; i<grouping.numberOfIObjectGroups(); i++)
				order += " "+(i+1);
			return order;
		}

		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				String order = textField.getText();
				String[] orders = order.split("[\\D]");
				print(orders);
				int[] intOrders = new int[orders.length];
				try{
					for(int i=0; i<orders.length; i++){
						intOrders[i]=Integer.parseInt(orders[i].trim())-1;
						if(intOrders[i]<0 || intOrders[i]>= grouping.numberOfIObjectGroups())
							throw new NumberFormatException();
					}
				}catch(NumberFormatException nfe){
					System.out.println("Incorrect order input!");
					return;
				}
				//		print(intOrders);
				grouping.reorder(intOrders);
				grouping.sort();


				order = getOrderString();
				textField.setText(order);

				view.resetImage();
				view.forceChange();
				model.update();
			}

		}

		public void print(String[] s){
			for(int i=0; i<s.length; i++){
				System.out.print(s[i]+" ");
			}
			System.out.println();
		}

		public void print(int[] s){
			for(int i=0; i<s.length; i++){
				System.out.print(s[i]+" ");
			}
			System.out.println();
		}


	}


	/***************************************************/
	/*
    private class Groups extends JPanel{

    	IObjectGrouping grouping;
    	JPanel p;
    	JScrollPane sp;

    	public Groups(IObjectGrouping grouping){
    	    this.grouping=grouping;
    	    p = new JPanel();

    	    Vector<IObjectGroup> groups = grouping.getGroups();
    	    p.setLayout(new GridLayout(groups.size(), 1));

    	    for(int i=0; i<groups.size(); i++){
    		//		GroupInfo gi = new GroupInfo(this, i+1, groups.elementAt(i));
    		GroupInfo gi = new GroupInfo(i+1, groups.elementAt(i));
    		p.add(gi);
    	    }

    	    sp = new JScrollPane(p);
    	    sp.setViewportView(p);
    	    //sp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    	    add(sp);
    	}

    	public void resetViewportView(){
    		sp.setViewportView(p);
    	}

    	public void paintComponent(Graphics g) {
    	    super.paintComponent(g);
    	    //resetViewportView();
    	    //System.out.println("Groups paint!");
    	}

        }
	 */

	private class Groups extends JScrollPane{

		private IObjectGrouping grouping;
		private boolean colorPanel;
		private boolean rowPanel;

		public Groups(IObjectGrouping grouping, boolean rowPanel, boolean colorPanel){
			this.grouping=grouping;
			this.rowPanel=rowPanel;
			this.colorPanel=colorPanel;
			JPanel p = new JPanel();

			Vector<IObjectGroup> groups = grouping.getGroups();
			p.setLayout(new GridLayout(groups.size(), 1));

			int height=0;
			int width=0;
			for(int i=0; i<groups.size(); i++){
				//		GroupInfo gi = new GroupInfo(this, i+1, groups.elementAt(i));
				GroupInfo gi = 
					new GroupInfo(view, i+1, grouping, groups.elementAt(i), 
							rowPanel, colorPanel);
				height+=gi.getPreferredSize().getHeight();
				width+=gi.getPreferredSize().getWidth();
				p.add(gi);
			}

			setPreferredSize(new Dimension(width, height));
			setBorder(new LineBorder(Color.BLACK));
			setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

			/*
			Dimension d = new Dimension(10, (int)getPreferredSize().getHeight());
			getVerticalScrollBar().setPreferredSize(d);
			getVerticalScrollBar().setMinimumSize(d);
			getVerticalScrollBar().setMaximumSize(d);
			getVerticalScrollBar().setSize(d);
			*/
			
			setViewportView(p);
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);
		}

	}

	/***************************************************/
	private class GroupInfo extends JPanel
	implements ActionListener{

		private InfomatView view;
		private IObjectGrouping grouping;
		private IObjectGroup group;
		private int order;
		private boolean colorPanel;
		private boolean rowPanel;
		//	ShowGroupingSettings sgs;
		private JTextField textField;
		private JButton newColorButton;
		private JButton currentColorButton;

		public GroupInfo(InfomatView view,
				int order, IObjectGrouping grouping, IObjectGroup group, 
				boolean rowPanel, boolean colorPanel){
			this.grouping=grouping;
			this.group=group;
			this.view=view;
			this.order=order;
			this.colorPanel=colorPanel;
			this.rowPanel=rowPanel;

			if(colorPanel){
				setLayout(new GridLayout(3,1));
			}
			else
				setLayout(new GridLayout(2,1));

			putAll();
			setBorder(new LineBorder(Color.BLACK));

			/*
	    Dimension d= new 
	    	Dimension((int) textField.getPreferredSize().getWidth(), 
	    				ShowGroupingSettings.textHeight);
	    //Dimension d = textField.getPreferredSize();
	    setPreferredSize(d);
	    setMinimumSize(getPreferredSize());
	    setMaximumSize(getPreferredSize());
			 */
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);

			/*
	    if(colorPanel){
		    GroupingColoring gc = view.getColoringManager().getColoring(rowPanel);
	    	Color color = gc.getColorForGroup(group); 

	    	currentColorLabel.setForeground(color);
	    	currentColorLabel.setBackground(color);

	    	Graphics g2 = currentColorLabel.getGraphics();
	    	Dimension d = currentColorLabel.getSize();
	    	g2.setColor(color);
	    	g2.fillRect(0, 0, (int)d.getWidth(), (int)d.getHeight());
	    }
			 */
		}

		public void putAll(){
			removeAll();
			int height = (int) (new JTextField("Dummy")).getPreferredSize().getHeight();
			// Name
			String name = ""+order+". "+group.getName()+"  ("+group.numberOfIObjects();
			//String name = "<html><h3>"+order+". "+group.getName()+"  ("+group.size()+")</h3></html>";
			if(rowPanel)
				name += " rows)";
			else
				name += " columns)";
			JLabel label = new JLabel(name);
			//label.setFont(new Font(Font.BOLD));
			add(label);
			//int height = (int) label.getPreferredSize().getHeight();
			int nr = String.valueOf(order).length()+2;
			String start = "";
			for(int i=0; i<nr; i++)
				start+=" ";

			// Color
			if(colorPanel){
				JPanel colorPanel = new JPanel();
				colorPanel.setLayout(new GridLayout(1,3));

				JLabel l = new JLabel(start+"Color:");
				colorPanel.add(l);

				GroupingColoring gc = 
					view.getGroupingManager(rowPanel).getCurrentColoring();
				/*
				GroupingColoring gc = 
					view.getColoringManager().getColoring(rowPanel);
				*/
				if(gc==null){
					colorPanel.add(new JLabel("No color"));
				}else{
				Color color = gc.getColorForGroup(group); 
				currentColorButton = new JButton("Current");
				currentColorButton.setBackground(color);
				Dimension d = new Dimension((int)currentColorButton.getPreferredSize().getWidth(), height);
				currentColorButton.setPreferredSize(d);
				currentColorButton.setMaximumSize(d);
				currentColorButton.setMinimumSize(d);

				colorPanel.add(currentColorButton);

				Color newColor = gc.getNewColorForGroup(group); 
				newColorButton = new JButton("New");
				newColorButton.setBackground(newColor);
				newColorButton.addActionListener(this);
				d=new Dimension((int)newColorButton.getPreferredSize().getWidth(), height);
				newColorButton.setPreferredSize(d);
				newColorButton.setMaximumSize(d);
				newColorButton.setMinimumSize(d);
				colorPanel.add(newColorButton);
				}
				add(colorPanel);
			}

			// Change Name
			JPanel part = new JPanel();

			//part.setLayout(new GridLayout(1,2));
			part.setLayout(new BoxLayout(part, BoxLayout.X_AXIS));

			label = new JLabel(start+"Change Name:");
			part.add(label);
			textField = new JTextField(10);
			textField.setText(group.getName());
			//Dimension d = new Dimension((int) textField.getPreferredSize().getWidth(), height);
			//textField.setPreferredSize(d);
			part.add(textField);
			//d = new Dimension((int) part.getPreferredSize().getWidth(), height);
			//part.setPreferredSize(d);

			add(part);

			/*
		height = (int) textField.getPreferredSize().getHeight();
		int width = (int) textField.getPreferredSize().getWidth();
		int heightMult=2;
		if(colorPanel)
			heightMult=3;
		Dimension d = new Dimension(2*width, heightMult*height);
			 */
			Dimension d = getPreferredSize();
			setPreferredSize(d);
			setMinimumSize(d);
			setMaximumSize(d);

			/*
	    d=getPreferredSize();
	    setMaximumSize(d);
	    setMinimumSize(d);
			 */

			textField.addActionListener(this);
			revalidate();
			repaint();
		}

		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JTextField){    	    
				String text = textField.getText();
				//System.out.println("-------------Change: "+text);
				group.setName(text);
			}else if(e.getSource() instanceof JButton){
				JButton jb = (JButton) e.getSource();
				if(jb==newColorButton){
					GroupingColoring gc = 
						view.getGroupingManager(rowPanel).getCurrentColoring();
					/*
					GroupingColoring gc = 
						view.getColoringManager().getColoring(rowPanel);
					*/
					Color newColor = gc.getNewColorForGroup(group); 
					newColor = 
						JColorChooser.showDialog(this,
								"Choose New Color for Group"+group.getName(),
								newColor);
					gc.setNewColorForGroup(newColor, group);
				}    		
			}
			putAll();
		}


	}

	/***************************************************/
	private class ColorChanger extends JPanel
	implements ActionListener{

		private InfomatView view;
		private IObjectGrouping grouping;
		private boolean rowPanel;

		private JButton applyButton;
		private JButton resetButton;

		public ColorChanger(InfomatView view, IObjectGrouping grouping, 
				boolean rowPanel){
			this.view=view;
			this.grouping=grouping;
			this.rowPanel=rowPanel;

			JLabel label = new JLabel("Coloring: ");
			add(label);

			applyButton = new JButton("Apply");
			applyButton.addActionListener(this);
			add(applyButton);

			resetButton = new JButton("Reset");
			resetButton.addActionListener(this);
			add(resetButton);	    

			int height = (int) (new JTextField("Dummy")).getPreferredSize().getHeight();
			applyButton.setPreferredSize(new Dimension((int)applyButton.getPreferredSize().getWidth(), height));
			resetButton.setPreferredSize(new Dimension((int)resetButton.getPreferredSize().getWidth(), height));	    

			Dimension d = getPreferredSize(); //new Dimension(prefWidth, prefHeight);
			setPreferredSize(d);
			setMinimumSize(d);
			setMaximumSize(d);
		}


		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				JButton button = (JButton) e.getSource();
				if(button==applyButton){
					GroupingColoring gc = 
						view.getGroupingManager(rowPanel).getCurrentColoring();
					/*
					GroupingColoring gc = 
						view.getColoringManager().getColoring(rowPanel);
					*/
					gc.changeToNewColors();
					gc.recolor();
				}
				else if(button==resetButton){
					GroupingColoring gc = 
						view.getGroupingManager(rowPanel).getCurrentColoring();
					/*
					GroupingColoring gc = 
						view.getColoringManager().getColoring(rowPanel);
					*/
					gc.resetColors();
				}

				view.forceChange();
				model.update();
			}
		}

	}



}
