package infomat.view.info;

import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import infomat.vectorspace.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

public class PixelInfoPanel extends JPanel{

	private InfomatModel model;
	private InfomatView view;

	private IObject currentRowObject;
	private IObject currentColObject;
	
	private TitledBorder titleBorder;
	
	//private CellPanel highestCell;
	private CellsPanel cellsPanel;
	
	public PixelInfoPanel(InfomatModel model, InfomatView view){
		this.model=model;
		this.view = view;
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		titleBorder = new TitledBorder("Pixels");
		setBorder(titleBorder);
		
		//highestCell = new CellPanel();
		cellsPanel = new CellsPanel();
		cellsPanel.setLayout(new BoxLayout(cellsPanel, BoxLayout.Y_AXIS));
		
		putAll();
		repaint();
	}
	
	
	
	public void putAll(){
		removeAll();
//		add(highestCell);
		add(cellsPanel);
		revalidate();
		repaint();
	}
	
	/*
	public void setShowAllPixels(){
		cellsPanel.one=false;
	}
	*/
	
	public void setCurrentInfo(IObject rowObject, IObject colObject){
		currentRowObject=rowObject;
		currentColObject=colObject;
		
		SparsePicture sparsePicture = view.getSparsePicture();
		SparsePixel sp = sparsePicture.getPixelForObjects(rowObject, colObject);
		//System.out.println("SparsePixel: "+sp);
		if(sp!=null){
			//IMatrixCell mcHeighest =sp.getIMatrixCellWithHighestWeight(); 
			//highestCell = new CellPanel(mcHeighest);
			cellsPanel.setCurrentInfo(sp);
		}else{
			//highestCell = new CellPanel();
			cellsPanel.setCurrentInfo(null);
		}
		putAll();
		
		/*
		if(matrix!=null){
			if(sp!=null){
				//sp.get
			}
		}	
		*/	
	}
	
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
	}
	

	private class CellsPanel extends JPanel
	implements ActionListener{
		
		private SparsePixel currentPixel;
		private boolean one=false;
		private JButton oneButton;
		//private JButton allButton;
		private JPanel buttonPanel;
		
		private JScrollPane cellsScrollPane;
		private JPanel scrollPanel;
		
		public CellsPanel(){
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			
			JTextField dummy = new JTextField("OneTwoT");
			Dimension d = dummy.getPreferredSize();

			oneButton = new JButton("All");
			oneButton.setPreferredSize(d);
			oneButton.setMinimumSize(d);
			oneButton.setMaximumSize(d);
			oneButton.addActionListener(this);
			
			/*
			buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			oneButton = new JButton("One");
			oneButton.setPreferredSize(d);
			oneButton.addActionListener(this);
			allButton = new JButton("All");
			allButton.setPreferredSize(d);
			allButton.addActionListener(this);
			buttonPanel.add(oneButton);
			buttonPanel.add(allButton);
			*/
			
			scrollPanel = new JPanel();
			scrollPanel.setLayout(new BoxLayout(scrollPanel, BoxLayout.Y_AXIS));
			cellsScrollPane = new JScrollPane();
			//cellsScrollPane.setLayout(new BoxLayout(cellsScrollPane, BoxLayout.X_AXIS));
			cellsScrollPane.setBorder(new LineBorder(Color.BLACK));
			cellsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			cellsScrollPane.setViewportView(scrollPanel);
			putAll();
		}
		
		public void putAll(){
			removeAll();
			//add(buttonPanel);
			add(oneButton);
			
			add(cellsScrollPane);
			revalidate();
			repaint();
		}

		public void paintComponent(Graphics g){
			super.paintComponent(g);
		}

		public void setCurrentInfo(SparsePixel sp){
			currentPixel=sp;
			scrollPanel.removeAll();
			if(sp!=null){
				if(one){
					IMatrixCell highest = sp.getIMatrixCellWithHighestWeight();
					scrollPanel.setLayout(new GridLayout(1, 1));
					CellPanel cellPanel = new CellPanel(highest);
					//Dimension d = new Dimension(getSize().width, cellPanel.getSize().height);
					scrollPanel.add(cellPanel);
				}else{
					Vector<IMatrixCell> cells = sp.getAllIMatrixCells();
					scrollPanel.setLayout(new GridLayout(cells.size(), 1));
					//int width = getSize().width;
					for(int i=0; i<cells.size(); i++){
						CellPanel cellPanel = new CellPanel(cells.elementAt(i));
						//Dimension d = new Dimension(width, cellPanel.getSize().height);
						scrollPanel.add(cellPanel);
					}
				}
			}else{
				scrollPanel.setLayout(new GridLayout(1, 1));
				scrollPanel.add(new CellPanel());
			}
			putAll();
		}
		
		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				JButton button = (JButton) e.getSource();
				if(button==oneButton){
					//System.out.println("One");
					//one=true;
					one=!one;
					if(one){
						oneButton.setText("One");
					}else{
						oneButton.setText("All");
					}
					setCurrentInfo(currentPixel);
				}
				/*
				else if(button==allButton){
					//System.out.println("All");
					one=false;
				}
				*/	
			}
		}
	}
	
	
	private class CellPanel extends JPanel{
		
		private JLabel weightLabel;
		private JLabel rowLabel;
		private JLabel colLabel;
		
		
		public CellPanel(){
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(BorderFactory.createLineBorder(Color.black));
			weightLabel = new JLabel(" ");
			rowLabel = new JLabel(" ");
			colLabel = new JLabel(" ");
			add(weightLabel);
			add(rowLabel);
			add(colLabel);
			revalidate();
			repaint();
		}
		
		public CellPanel(IMatrixCell mc){
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(BorderFactory.createLineBorder(Color.black));
			weightLabel = new JLabel("w: "+mc.getWeight());
			rowLabel = new JLabel("r:   "+mc.getRowIObject().getString());
			colLabel = new JLabel("c:  "+mc.getColumnIObject().getString());
			add(weightLabel);
			add(rowLabel);
			add(colLabel);
			revalidate();
			repaint();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
		}
	
		
	}
	
}
