package infomat.view.menu;

import infomat.view.*;

import java.io.*;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

/**
 * Responsible for the menu. Set it up and listen for events.
 */
public class Menu extends JMenuBar implements ActionListener {

	private MainWindow mainWindow;

	private InfomatModel infomatModel;

	/************************************************/
	private final static String FILE = "File";
	private final static String OPENCORPUSFILE = "Open Corpus File";
	private final static String OPENDOCUMENTGROUPINGFILE = 
		"Open Document Grouping File";
	private final static String OPENMATRIXFILE = "Open IMatrix File";
	private final static String SAVEMATRIXFILE = "Save IMatrix File";

	private final static String SAVEMATRIXPICTURE = "Save IMatrix Picture";
	private final static String SAVEPICTURE = "Save Picture";
	private final static String SAVESESSION = "Save Session";
	private final static String LOADSESSION = "Load Session";
	private final static String EXIT = "Exit";

	/************************************************/
	private final static String IMAGE = "Image";

	private final static String COLOURING = "Colouring";
	private final static String GRIDCOLUMN = "Toggle Column Grid";
	private final static String GRIDROW = "Toggle Row Grid";

	//private final static String NCOLUMNPOINT = "Select # Columns at Point";
	//sprivate final static String NROWPOINT = "Select # Rows at Point";
	private final static String REMOVE_COLUMNS = "Remove Columns";
	private final static String REMOVE_ROWS = "Remove Rows";

	private final static String PIXELSELECTION= "Pixel Selection";
	private final static String MOVE = "Drag";
	private final static String ZOOMSELECTION = "Zoom Selection";
	private final static String ZOOM = "Zoom";

	private final static String RESET = "Reset zoom";

	/************************************************/
	private final static String WINDOWS = "Windows";
	private final static String TOOLBARONOFF = "Toolbar On/Off";
	//private final static String INFOPANELONOFF = "Info Panel On/Off";
	private final static String GROUPINGPANELONOFF = "Grouping Panel On/Off";
	private final static String OVERVIEWONOFF = "Overview On/Off";
	private final static String PIXELINFOONOFF = "Pixel Info On/Off";

	/************************************************/
	private final static String TOOLS = "Tools";
	private final static String INVERTMATRIX = "Invert Matrix";
	private final static String PURGEMATRIX = "Purge Matrix";

	/************************************************/
	private final static String ALGORITHMS = "Algorithms";
	private final static String KMEANS = "K-Means";
	private final static String RELATIVECLUSTERING = "Relative Clustering";
	private final static String RANDOMCLUSTERING = "Random Clustering";
	//private final static String DENSITY = "Density";

	/************************************************/
	private final static String HELP = "Help";
	private final static String ABOUT = "About";
	private final static String MANUAL = "Manual";


	private ToolbarController toolbarController;

	private InfomatView infomatView;

	private JFileChooser fileChooser;
	private JFileChooser imgFileChooser;

	private String fileChooserPath=""; //"E:/Java/Infomat/example/"; //"/var/tmp/rosell/Result/xml/";

	/**
	 * Create the menu.
	 * 
	 * @param frame
	 *            The frame to dispose on exit.
	 */
	public Menu(MainWindow frame, InfomatModel infoModel,
			ToolbarController toolbarController, InfomatView infomatView) {
		this.toolbarController = toolbarController;
		this.infomatView = infomatView;
		infomatModel = infoModel;
		mainWindow = frame;

		fileChooserPath = (String) infomatModel.getPropertyValue("Matrix File Path");
		fileChooser = new JFileChooser(fileChooserPath);
		fileChooser.setFileFilter(new FileFilter() {
			public String getDescription() {
				return "XML files";
			}

			public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				int findresult = f.getName().indexOf(".xml");
				int len = f.getName().length();
				return findresult != -1 && findresult == len - 4;
			}

		});

		imgFileChooser = new JFileChooser(fileChooserPath);
		imgFileChooser.setFileFilter(new FileFilter() {
			public String getDescription() {
				return "PNG files";
			}

			public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				int findresult = f.getName().indexOf(".png");
				int len = f.getName().length();
				return findresult != -1 && findresult == len - 4;
			}

		});


		JMenu menu;
		JMenuItem menuItem;

	    /************************************************/		
		// File menu
		menu = new JMenu(FILE);
		menu.setMnemonic(KeyEvent.VK_F);
		add(menu);

		menuItem = new JMenuItem(OPENMATRIXFILE, KeyEvent.VK_O);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(SAVEMATRIXFILE, KeyEvent.VK_O);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		
		menu.addSeparator();

		menuItem = new JMenuItem(OPENCORPUSFILE, KeyEvent.VK_O);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(OPENDOCUMENTGROUPINGFILE, KeyEvent.VK_O);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		
		menu.addSeparator();

		menuItem = new JMenuItem(SAVEMATRIXPICTURE, KeyEvent.VK_P);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		

		
		/*
	  menuItem = new JMenuItem(SAVEPICTURE, KeyEvent.VK_P);
	  menuItem.addActionListener(this);
	  menuItem.setEnabled(false);
	  menu.add(menuItem);

	  menu.addSeparator();

	  menuItem = new JMenuItem(SAVESESSION, KeyEvent.VK_S);
	  menuItem.addActionListener(this);
	  menuItem.setEnabled(false);
	  menu.add(menuItem);

	  menuItem = new JMenuItem(LOADSESSION, KeyEvent.VK_L);
	  menuItem.addActionListener(this);
	  menuItem.setEnabled(false);
	  menu.add(menuItem);
		 */

		menu.addSeparator();

		menuItem = new JMenuItem(EXIT, KeyEvent.VK_X);
		menuItem.addActionListener(this);
		menu.add(menuItem);

	    /************************************************/
		// Image menu
		menu = new JMenu(IMAGE);
		menu.setMnemonic(KeyEvent.VK_I);
		add(menu);

		/*
		menuItem = new JMenuItem(TOOLBARONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		*/
		/*
	  menuItem = new JMenuItem(COLOURING, KeyEvent.VK_C);
	  menuItem.addActionListener(this);
	  menuItem.setEnabled(false);
	  menu.add(menuItem);
		menu.addSeparator();
		 */

		menuItem = new JMenuItem(PIXELSELECTION, KeyEvent.VK_M);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(MOVE, KeyEvent.VK_M);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(ZOOMSELECTION, KeyEvent.VK_S);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(ZOOM, KeyEvent.VK_Z);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		menu.addSeparator();

		menuItem = new JMenuItem(GRIDCOLUMN, KeyEvent.VK_W);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		/*
		menuItem = new JMenuItem(NCOLUMNPOINT, KeyEvent.VK_P);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		*/
		
		menuItem = new JMenuItem(REMOVE_COLUMNS, KeyEvent.VK_E);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menu.addSeparator();

		menuItem = new JMenuItem(GRIDROW, KeyEvent.VK_D);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		/*
		menuItem = new JMenuItem(NROWPOINT, KeyEvent.VK_S);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		 */
		
		menuItem = new JMenuItem(REMOVE_ROWS, KeyEvent.VK_M);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menu.addSeparator();

		menuItem = new JMenuItem(RESET, KeyEvent.VK_R);
		menuItem.addActionListener(this);
		menu.add(menuItem);

	    /************************************************/
		// Panel menu
		menu = new JMenu(WINDOWS);
		menu.setMnemonic(KeyEvent.VK_I);
		add(menu);

		menuItem = new JMenuItem(TOOLBARONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		/*
		menuItem = new JMenuItem(INFOPANELONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		 */
		
		menuItem = new JMenuItem(GROUPINGPANELONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(PIXELINFOONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		
		menuItem = new JMenuItem(OVERVIEWONOFF, KeyEvent.VK_T);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		
		
		
		/************************************************/
		// Tools menu
		menu = new JMenu(TOOLS);
		menu.setMnemonic(KeyEvent.VK_H);
		add(menu);

		menuItem = new JMenuItem(INVERTMATRIX, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(PURGEMATRIX, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);

	    /************************************************/
		// Algorithm menu
		menu = new JMenu(ALGORITHMS);
		menu.setMnemonic(KeyEvent.VK_H);
		add(menu);

		menuItem = new JMenuItem(KMEANS, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		
		
		menuItem = new JMenuItem(RELATIVECLUSTERING, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(RANDOMCLUSTERING, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		/*
		menuItem = new JMenuItem(DENSITY, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);
		 */
		
	    /************************************************/
		// Help menu
		menu = new JMenu(HELP);
		menu.setMnemonic(KeyEvent.VK_H);
		add(menu);

		menuItem = new JMenuItem(ABOUT, KeyEvent.VK_A);
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menuItem = new JMenuItem(MANUAL, KeyEvent.VK_M);
		menuItem.addActionListener(this);
		menu.add(menuItem);
	}

	public void actionPerformed(ActionEvent e) {
		JMenuItem source = (JMenuItem) (e.getSource());
		String cmd = source.getText();
		String s = "Menu action event detected." + "    Event source: "
		+ source.getText();
		System.out.println(s);

	    /************************************************/
		// File menu
		if (cmd.equals(EXIT)) {
			mainWindow.getMainFrame().dispose();
			//infomatView.disposeOverviewFrame();
		} else if (cmd.equals(OPENCORPUSFILE)) {
			fileChooser.setSelectedFile(new File(""));
			if (fileChooser.showOpenDialog(mainWindow.getSessionFrame()) == JFileChooser.APPROVE_OPTION) {
				infomatModel.openCorpusFile(fileChooser.getSelectedFile());
			}	    
		} else if (cmd.equals(OPENDOCUMENTGROUPINGFILE)) {
			fileChooser.setSelectedFile(new File(""));
			if (fileChooser.showOpenDialog(mainWindow.getSessionFrame()) == JFileChooser.APPROVE_OPTION) {
				infomatModel.openDocumentGroupingFile(fileChooser.getSelectedFile());
				infomatView.forceChange();
				System.out.println("ppnat!!!");
			}
		} else if (cmd.equals(OPENMATRIXFILE)) {
			fileChooser.setSelectedFile(new File(""));
			if (fileChooser.showOpenDialog(mainWindow.getSessionFrame()) == JFileChooser.APPROVE_OPTION) {
				infomatModel.openIMatrixFile(fileChooser.getSelectedFile());
				System.out.println("ppnat!!!");
			}
		} else if (cmd.equals(SAVEMATRIXFILE)) {
			fileChooser.setSelectedFile(new File(""));
			if (fileChooser.showOpenDialog(mainWindow.getSessionFrame()) == JFileChooser.APPROVE_OPTION) {
				infomatModel.saveIMatrixFile(fileChooser.getSelectedFile());
				System.out.println("Sparat!!!");
			}
		} else if (cmd.equals(SAVEMATRIXPICTURE)) {
			imgFileChooser.setSelectedFile(new File(""));
			if (imgFileChooser.showOpenDialog(mainWindow.getSessionFrame()) == JFileChooser.APPROVE_OPTION) {
				//		infomatModel.openDocumentGroupingFile(fileChooser.getSelectedFile());

				System.out.println("SAVEMATRIXPICTURE!!!");
				System.out.println(fileChooser.getSelectedFile().getName());
				infomatModel.saveIMatrixPicture(imgFileChooser.getSelectedFile());
			}
		} 

	    /************************************************/
		// Image menu
		else if (cmd.equals(RESET)) {
			System.out.println("Reset image position and zoom");
			infomatModel.resetImage();
		} else if (cmd.equals(PIXELSELECTION)) {
			System.out.println("Click the hand");
			toolbarController.menuClick(ToolbarController.PIXELSELECTION);
		} else if (cmd.equals(MOVE)) {
			System.out.println("Click the hand");
			toolbarController.menuClick(ToolbarController.HAND);
		} else if (cmd.equals(ZOOMSELECTION)) {
			System.out.println("Click the zoomselection");
			toolbarController.menuClick(ToolbarController.ZOOMSELECTION);
		} else if (cmd.equals(ZOOM)) {
			System.out.println("Click the zoom");
			toolbarController.menuClick(ToolbarController.ZOOMIN);
		} else if (cmd.equals(GRIDCOLUMN)) {
			System.out.println("Click the word grid");
			toolbarController.menuClick(ToolbarController.GRIDCOLUMN);
		} else if (cmd.equals(GRIDROW)) {
			System.out.println("Click the document grid");
			toolbarController.menuClick(ToolbarController.GRIDROW);
		} else if (cmd.equals(REMOVE_COLUMNS)) {
			toolbarController.menuClick(ToolbarController.REMOVE_COLUMNS);
		} else if (cmd.equals(REMOVE_ROWS)) {
			toolbarController.menuClick(ToolbarController.REMOVE_ROWS);
		} else if (cmd.equals(SAVEPICTURE)) {
			System.out.println("Click the save");
			// toolbarController.menuClick(ToolbarController.SAVE);
		} 
		/*
		else if (cmd.equals(NCOLUMNPOINT)) {
			//	    infomatView.askForWordCount();
		} else if (cmd.equals(NROWPOINT)) {
			//	    infomatView.askForDocumentsCount();
		} 
		*/
		
	    /************************************************/
		// Panel menu
		else if (cmd.equals(TOOLBARONOFF)) {
			mainWindow.getSessionFrame().toggleToolbar();
		//}else if (cmd.equals(INFOPANELONOFF)) {
		//	mainWindow.getSessionFrame().toggleInfoPanel();
		}else if (cmd.equals(GROUPINGPANELONOFF)) {
			mainWindow.getSessionFrame().toggleGroupingPanel();
		}else if (cmd.equals(PIXELINFOONOFF)) {
			mainWindow.getSessionFrame().togglePixelInfo();
		}else if (cmd.equals(OVERVIEWONOFF)) {
			mainWindow.getSessionFrame().toggleOverview();
		}
		
		
		/************************************************/
		// Tools menu
		else if (cmd.equals(INVERTMATRIX)) {
			System.out.println("INVERT MATRIX");
			infomatModel.invertMatrix();
		} else if (cmd.equals(PURGEMATRIX)) {
			System.out.println("PURGE MATRIX");
			infomatModel.purgeMatrix();
		} 

		
		/************************************************/
		// Algorithm menu
		else if (cmd.equals(KMEANS)) {
			infomatModel.kmeans();
		}else if (cmd.equals(RELATIVECLUSTERING)) {
			infomatModel.relativeClustering();
		}else if (cmd.equals(RANDOMCLUSTERING)) {
			infomatModel.randomClustering();
		}
		/*
		} else if (cmd.equals(DENSITY)) {
			infomatModel.density();
		} 
		*/
		
		/************************************************/
		// Help menu
		else if (cmd.equals(ABOUT)) {
			JOptionPane
			.showMessageDialog(
					this,
					"Infomat "
					+ MainWindow.VERSION
					+ " - A Vector Space Visualization Tool.\n\n(C) 2007 Magnus Rosell, KTH CSC\n\nthanks to\nJoel Jonsson, Fredrika Wahlstrm, Urban Koistinen, Janna Iljina ",
					"About Infomat",
					JOptionPane.INFORMATION_MESSAGE);
		} else if (cmd.equals(MANUAL)) {
			JOptionPane.showMessageDialog(this, "The manual is a html-document in the Infomat directory.",
					"Infomat Interface Manual",
					JOptionPane.INFORMATION_MESSAGE);
		} 
		
		/************************************************/
		else {
			System.err.println("Menu action not definied! " + source.getText());
		}
	}
}
