package mro.util;

import java.util.*; 

/**
 * A DoubleHashMap has two indexes. 
 * There is only one copy of each index value.
 *
 * @version          061219
 * @author           Magnus Rosell
 */
public class DoubleHashMap{

	// The indexes are only saved one time
	/*
	private HashMap firstIndexMap = new HashMap(); 
	private HashMap secondIndexMap = new HashMap();
	 */
	private HashSet firstIndexSet = new HashSet();
	private HashSet secondIndexSet = new HashSet();
	
	// The Map, i.e. the first HashMap, which contains several HashMaps.
	private HashMap firstMap = new HashMap();

	// The number of Objects in the DoubleHashMap
	private int size=0;

	/**
	 * Adds a Object "value" indexed with the Object:s
	 * "first" and "second"
	 * @param first
	 * @param second
	 * @param value
	 */
	public void put(Object first, Object second, Object value){
		/*
		first = getIndexObject(first, firstIndexMap);
		second = getIndexObject(second, secondIndexMap);
		 */
		firstIndexSet.add(first);
		secondIndexSet.add(second);
		
		HashMap secondMap = (HashMap) firstMap.get(first);
		if(secondMap==null){
			secondMap = new HashMap();
			firstMap.put(first, secondMap);
		}
		secondMap.put(second, value);

		size++;
	}


	/**
	 * Returns the Object that was indexed with "first" and "second".
	 * @param first
	 * @param second
	 * @return an Object
	 */
	public Object get(Object first, Object second){
		HashMap secondMap = (HashMap) firstMap.get(first);
		if(secondMap==null)
			return null;
		Object value = secondMap.get(second);
		if(value==null)
			return null;
		return value;
	}

	/**
	 * Removes the Object with the indexes.
	 * @param first
	 * @param second
	 * @return an Object
	 */
	public Object remove(Object first, Object second){
		HashMap secondMap = (HashMap) firstMap.get(first);
		if(secondMap==null)
			return null;
		Object value = secondMap.remove(second);
		if(value==null)
			return null;
		if(secondMap.size()==0)
			firstMap.remove(secondMap);
		size--;
		return value;
	}

	/**
	 * Removes all Objects with the specified "first" index.
	 * @param first
	 */
	public void removeAllFirst(Object first){
		HashMap removed = (HashMap) firstMap.remove(first);
		size-=removed.size();
	}

	/**
	 * Removes all Objects with the specified "second" index.
	 * @param second
	 */
	public void removeAllSecond(Object second){
		Iterator it = firstMap.values().iterator();
		while(it.hasNext()){
			HashMap secondMap = (HashMap) it.next();
			secondMap.remove(second);
			size--;
		}
	}

	/**
	 * Returns all Objects in the DoubleHashMap as a Vector.
	 * @return a Vector
	 */
	public Vector values(){
		Vector ret = new Vector();
		Iterator it = firstMap.values().iterator();
		while(it.hasNext()){
			HashMap secondMap = (HashMap) it.next();
			Iterator it2 = secondMap.values().iterator();
			while(it2.hasNext())
				ret.add(it2.next());
		}
		return ret;
	}

	/**
	 * Returns a HashSet of those value Objects that are 
	 * stored with the specified "first" index.
	 * <br>
	 * This is the optimal method. Much faster than 
	 * #valuesForSecondIndex().
	 * @param first
	 * @return a Vector
	 */	
	public Vector valuesForFirstIndex(Object first){
		HashMap second = (HashMap) firstMap.get(first);
		if(second==null)
			return null;
		return new Vector(second.values());
	}

	public Vector valuesForSecondIndex(Object second){
		Vector ret = new Vector();
		Iterator it = firstMap.values().iterator();
		while(it.hasNext()){
			HashMap secondMap = (HashMap) it.next();
			Object o = secondMap.get(second);
			if(o!=null)
				ret.add(o);
			
			/*
			HashMap secondMap = (HashMap) it.next();
			Iterator it2 = secondMap.values().iterator();
			while(it2.hasNext()){
				Object o = it2.next();
				if(second==o)
					ret.add(o);
			}
			*/
		}
		return ret;
	}


	public int size(){
		return size;
	}

	public int sizeOfFirstIndex(){
		//return firstIndexMap.size();
		return firstIndexSet.size();
	}

	public int sizeOfSecondIndex(){
		return secondIndexSet.size();
		//return secondIndexMap.size();
	}

	public int sizeOfSecondIndexForElementInFirst(Object first){
		HashMap second = (HashMap) firstMap.get(first);
		if(second==null)
			return 0;
		return second.size();
	}

	public Set keySetOfSecondIndexForElementInFirst(Object first){
		HashMap second = (HashMap) firstMap.get(first);
		if(second==null)
			return null;
		return second.keySet();
	}

	/*
	private Object getIndexObject(Object index, HashMap hMap){
		Object i = hMap.get(index);
		if(i==null)
			hMap.put(index, index);
		else
			index=i;
		return index;
	}
	*/


}
