package mro.util.properties;

import mro.util.*;

import java.io.*;
import java.util.*; 

/**
 * A property of some kind with description. Used in many objects
 * to make presentation of results easier.
 * <b>Handles String, boolean, int, double fairly well...</b> Not for anything
 * else, yet.
 *
 * @version          061127
 * @author           Magnus Rosell
 */

public class Property{
	String name;
	Object value;
	int kind = 0; /* 0 - String
					 1 - boolean
					 2 - integer
					 3 - double
					*/
	String propertyDescription;
	Vector valueDescriptions=null;

	public static boolean roundValue=false;
	static int roundFloatToNumOfDecimals=3;

	// Only used from PropertyGroup
	public Property(){}

	private Property(String name, Object value, String propertyDescription){
		this.name=name;
		this.value=value;
		this.propertyDescription=propertyDescription;
	}

	public Property(String name, String value, String propertyDescription){
		this(name, (Object) value, propertyDescription);
		kind=0;
	}

	public Property(String name, int value, String propertyDescription){
		this(name, new Integer(value), propertyDescription);
		kind=2;
	}

	public Property(String name, double value, String propertyDescription){
		this(name, new Double(value), propertyDescription);
		kind=3;
	}

	public Property(String name, boolean value, String propertyDescription){
		this(name, new Boolean(value), propertyDescription);
		kind=1;
	}

	/**
	 * Copy Constructor.
	 */
	public Property(Property p2){
		this.name=p2.name;
		this.value=p2.value;
		this.propertyDescription=p2.propertyDescription;
		this.valueDescriptions=p2.valueDescriptions;
	}


	/**
	 * To read from XML files...
	 */
	public Property(String name, String value, boolean string,
			String propertyDescription){
		Object o=null;
		if(string){
			o=value;
			kind=0;
		}
		else{
			try{o = (Integer) Integer.parseInt(value); kind=2;}
			catch(Exception e){
				try{o = (Double) Double.parseDouble(value); kind=3;}
				catch(Exception e2){
					try{o = (Boolean) Boolean.parseBoolean(value); kind=1;}
					catch(Exception e3){
						System.out.println("Unsupported Property Value!");
						System.exit(0);
					}
				}
			}
		}

		/*
	System.out.print(name+":"+value);
	if(o instanceof Integer)
	    System.out.println(" Integer");
	else if(o instanceof Float)
	    System.out.println(" Float");
	else if(o instanceof Boolean)
	    System.out.println(" Boolean");
	else
	    System.out.println(" !!! Inget !!!");
		 */

		this.name=name;
		this.value=o;
		this.propertyDescription=propertyDescription;
	}

	public void setValue(Object value){
		this.value=value;
	}

	public Object getValue(){
		return value;
	}

	/**
	 * Returns an int telling what kind of value. 0 - String, 1 - boolean, 2 - integer, 3 - double.
	 * @return an int
	 */
	public int getKind(){
		return kind;
	}
	
	public String getName(){
		return name;
	}

	public String getDescription(){
		return propertyDescription;
	}

	public void addValueDescription(PropertyValueDescription cpvd){
		if(valueDescriptions==null)
			valueDescriptions=new Vector();
		valueDescriptions.add(cpvd);
	}

	public void addVectorOfValueDescriptions(Vector vvd){
		if(valueDescriptions==null)
			valueDescriptions=new Vector();
		for(int i=0; i<vvd.size(); i++){
			valueDescriptions.add(vvd.elementAt(i));
		}
	}

	public String getAppropriateValueDescription(){
		for(int i=0; i<valueDescriptions.size(); i++){
			PropertyValueDescription pvd = (PropertyValueDescription)
			valueDescriptions.elementAt(i);
			if(value.equals(pvd.value))
				return pvd.valueDescription;
		}
		return "Unknown Property Value";
	}

	public Vector getAllValueDescriptions(){
		return valueDescriptions;
	}

	public void printXML(PrintWriter pw) throws IOException{
		pw.println("<p>");
		pw.print("<n>"+name+"</n>");
		if(value instanceof String)
			pw.println("<v><str>"+value+"</str></v>");
		else
			pw.println("<v>"+value+"</v>");
		pw.println("<d>"+propertyDescription+"</d>");
		if(valueDescriptions!=null){
			for(int i=0; i<valueDescriptions.size(); i++){
				((PropertyValueDescription) valueDescriptions.elementAt(i)).printXML(pw);
			}
		}

		pw.println("</p>");
	}




	/****************** OLD *****************/


	private String valueString(){
		String s="";
		if(roundValue && value instanceof Float){
			s=Round.roundDecimalsToTrimmedString(((Float) value).floatValue(),
					roundFloatToNumOfDecimals);

		}else
			s += ""+value;
		return s;
	}


	public String toHTMLString(){
		String s="";
		if(valueDescriptions==null){
			s += name+": "+valueString()+" <br />\n";
		}else{
			if(!getAppropriateValueDescription().equals("Unknown Property Value")){
				s += name+": "+getAppropriateValueDescription()+" <br />\n";
			}
			else
				s += name+": "+valueString()+" <br />\n";
		}
		return s;
	}

	public String toEvaluationFileString(){
		String s="";
		s += name+":\n"+valueString()+"\n";

		return s;
	}

	public String toInfoFileString(){
		String s="";
		s += name+":\n";
		s += propertyDescription+"\n";
		if(valueDescriptions!=null){
			for(int i=0; i<valueDescriptions.size(); i++){
				PropertyValueDescription pvd = (PropertyValueDescription) 
				valueDescriptions.elementAt(i);
				s+=" "+pvd.value+" - "+pvd.valueDescription+"\n";
			}
		}

		return s;
	}


}


