package mro.util.properties.dialog;

import mro.util.properties.*;

import javax.swing.*;
import javax.swing.border.LineBorder;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;


public class PropertiesPanel extends JPanel{


	private Properties properties;

	public PropertiesPanel(Properties properties){
		this.properties=properties;

		//setLayout(new GridLayout(properties.size(), 1));

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setPreferredSize(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE);
		
		Vector<Property> props = properties.getPropertyVector();
		for(int i=0; i<props.size(); i++){
			add(new PropertyPanel(props.elementAt(i)));
		}
		revalidate();
		repaint();
	}

	private class PropertyPanel extends JPanel
	implements ActionListener{

		private Property property;
		private JTextField newValueField;
		private JLabel valueLabel;
		
		public PropertyPanel(Property property){
			this.property=property;
			int numberOfParts = 3;
			Vector descriptions = property.getAllValueDescriptions();
			if(descriptions!=null)
				numberOfParts += 1; //descriptions.size();
			
			//setLayout(new GridLayout(numberOfParts, 1));
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new LineBorder(Color.BLACK));
			putAll();
		}

		private void putAll(){
			removeAll();
			JPanel headPanel = new JPanel();
			headPanel.setLayout(new GridLayout(3,1));
			//headPanel.setLayout(new BoxLayout(headPanel, BoxLayout.Y_AXIS));
			
			JLabel title = new JLabel(property.getName());
			headPanel.add(title);
			String value = (String) property.getValue().toString();
			valueLabel = new JLabel("Value: "+value);
			headPanel.add(valueLabel);

			JPanel n = new JPanel();
			n.setLayout(new BoxLayout(n, BoxLayout.X_AXIS));
			JLabel nl = new JLabel("New value: ");
			n.add(nl);
			/*
			String newValue = "";
			for(int i=0; i<value.length(); i++)
				newValue += "     ";
			*/
			newValueField = new JTextField(10); //newValue);
//			Dimension d = new Dimension(newValueField.getSize().width, nl.getSize().height);

			Dimension d = newValueField.getPreferredSize();
			newValueField.setPreferredSize(d);
			newValueField.setMaximumSize(d);
			newValueField.setMinimumSize(d);
			newValueField.addActionListener(this);

			n.add(newValueField);
			headPanel.add(n);
			int width = (int) headPanel.getPreferredSize().getWidth();
			d = new Dimension(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE.width, (int) (3*d.getHeight()));
			headPanel.setPreferredSize(d);
			//headPanel.setMinimumSize(d);
			//headPanel.setMaximumSize(d);
			
			add(headPanel);

			Vector descriptions = property.getAllValueDescriptions();
			if(descriptions!=null){
				//JScrollPane scroll = new JScrollPane();
				JPanel descriptionPanel = new JPanel();
				descriptionPanel.setLayout(new GridLayout(descriptions.size()+1,1));
				//descriptionPanel.setLayout(new BoxLayout(descriptionPanel, BoxLayout.Y_AXIS));
				descriptionPanel.add(new JLabel("Possible values:"));
				for(int i=0; i<descriptions.size(); i++)
					descriptionPanel.add(new PropertyValueDescriptionPanel((PropertyValueDescription) descriptions.elementAt(i)));
				//scroll.setViewportView(descriptionPanel);
				//add(scroll);
				add(descriptionPanel);
			}
			
			revalidate();
			repaint();

		}

		public void paintcomponent(Graphics g){
			super.paintComponent(g);
		}
		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JTextField){    	    
				String newText = newValueField.getText();
				//System.out.println("-------------Change: "+newText);
				boolean ok=true;
				
				Vector descriptions = property.getAllValueDescriptions();
				if(descriptions!=null){
					//System.out.println("Descriptions...");
					ok=false;
					for(int i=0; i<descriptions.size(); i++){
						PropertyValueDescription pvd =(PropertyValueDescription) descriptions.elementAt(i);
						if(newText.equals(pvd.getValue()))
							ok=true;
						if(property.getKind()==1){ // boolean
							if(newText.equals("true") ||newText.equals("false"))
								ok=true;
						}
					}
				}
				
				if(ok){
					//System.out.println("OK!");
					int kind = property.getKind();
					//System.out.println("kind: "+kind);
					if(kind==0){
						property.setValue(newText);
					}else if(kind==1){
						boolean b = Boolean.parseBoolean(newText);
						property.setValue(b);
					}else if(kind==2){
						try{
							int i = Integer.parseInt(newText);
							property.setValue(i);
						}catch(NumberFormatException nfe){
							// Do nothing
						}
					}else if(kind==3){
						try{
							double d = Double.parseDouble(newText);
							property.setValue(d);
						}catch(NumberFormatException nfe){
							// Do nothing
						}
					}
					
					putAll();
				}
			}
		}
	}

	private class PropertyValueDescriptionPanel extends JPanel{
		
		public PropertyValueDescriptionPanel(PropertyValueDescription pvd){
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			Object value = pvd.getValue();
			JLabel l = new JLabel("       * "+value + " - "+pvd.getValueDescription());
			add(l);
		}
	}
	
}
